# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2017 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#
# $Header: /project/isv-agent/VProAgents/WebSphere/WebSphere.pm,v 1.18 2015/03/13 11:35:48 apatne Exp $
#
# $Author: apatne $
# $Date: 2015/03/13 11:35:48 $
# $Revision: 1.18 $
#
#               Veritas Software Corporation
#
#
# Provides:                  WebSphere
# Author:                    spagare
# Creation Date:             20050313 (Sunday March 13, 22:10:03 PST 2005)
#
#
# Provides: package WebSphere
#
# Functionality: provides services for the WebSphere Application Server.
#-------------------------------------------------------------------------------

use strict;
use Comms;
use IO::Socket;
use POSIX;

#-------------------------
# Declare the package
#-------------------------
package WebSphere;

use POSIX qw ( uname );
use IO::File;

#-------------------------------------
# include required framework modules
#-------------------------------------
use SystemD;         # Support generalized SystemD Functions
use VCSagentFW;     # Primary Agent Interface to the ACC Library
use VCSDefines;     # This contains constants that VCS uses
use Sys;            # support generalized UNIX commands
use Version;        # Version display generator
use Proc;           # For calls to the process routines

my ( $CLUS_HOME, $sPrefix ) = ();

#------------------------
# Setup VCS/VCSone home..
#------------------------
if ( $ENV { CLUSTER_HOME } ) { # VCS 5.x/VCSOne..
  $CLUS_HOME ||= $ENV { CLUSTER_HOME } ||= "/opt/VRTSvcsone";
}
else { # VCS 4.x..
  $CLUS_HOME ||= $ENV { VCS_HOME } ||= "/opt/VRTSvcs";
}

( $sPrefix ) = $CLUS_HOME =~ /.*(vad|vcs(|one))$/;

#-----------------------------------------
# Define module-vendor specific commands..
#-----------------------------------------
my $sCmdHares  = $HARES  ||= "$CLUS_HOME/bin/hares";
my $sCmdHagrp  = $HAGRP  ||= "$CLUS_HOME/bin/hagrp";
my $sCmdHatype = $HATYPE ||= "$CLUS_HOME/bin/hatype";
my $sCmdHasys  = $HASYS  ||= "$CLUS_HOME/bin/hasys";
my $sCmdHacli  = "$CLUS_HOME/bin/hacli";
my $sCmdSSH    = "/usr/bin/ssh";
my $sCmdVcsDecrypt = $HADECRYPT ||= "$CLUS_HOME/sbin/${sPrefix}decrypt";

#---------------------------------------------------------------------------
# Add our version number to the version hash and check if versions are
# requested.
#---------------------------------------------------------------------------
Version::TellVersion("\$Id: WebSphere.pm,v 1.18 2015/03/13 11:35:48 apatne Exp $::ETVHeader");

use constant EPOP           => 10;
use constant CHECK_INTERVAL => 5;

#--------------------------------
# Define Application constants.
#--------------------------------

use constant WAS_STATUS        => "serverStatus.sh";
use constant WAS_STOP_NODE     => "stopNode.sh";
use constant WAS_STOP_SERVER   => "stopServer.sh";
use constant WAS_STOP_MANAGER  => "stopManager.sh";
use constant WAS_START_NODE    => "startNode.sh";
use constant WAS_START_SERVER  => "startServer.sh";
use constant WAS_START_MANAGER => "startManager.sh";
use constant WAS_WSADMIN       => "wsadmin.sh";

#-------------------------------------------------------------
# Define ps command and its arguments according to platform
#------------------------------------------------------------
my $sOSType        = Arch::GetOSType();
my $sCommandGrep   = "/bin/grep";
my $sCommandPs     = "/bin/ps";
my $sCommandPsArgs = "axwwl";

if ( Arch::SUN_OS == $sOSType ) {
  my $sOSVer = ( uname ) [2]; # Get OS version..
    if ($sOSVer =~ /\d+\.(\d+)/) {
      $sOSVer =$1;
    }
  $sCommandPs = "/usr/bin/ps";
  if ( $sOSVer < 11 ) {
    $sCommandPs = "/usr/ucb/ps" if ( -f "/usr/ucb/ps" );
  }
}
elsif ( Arch::HPUX_OS == $sOSType ) {
  $sCommandPsArgs = "-e -xxo uid,pid,ppid,args";
}
elsif ( Arch::LINUX_OS == $sOSType ) {
  $sCommandPsArgs = "ww -eo uid,pid,ppid,args"
}
elsif ( Arch::AIX_OS == $sOSType ){
    $sCommandPsArgs = "-e -o 'uid,pid,ppid,args'";
}

=pod

--------------------------------------------------------------------------------

 Function : Boolean ArgsValid (IN String    sEntryPointName,
                               IN String    localWASHome,
                               IN String    localWASNode,
                               IN String    localUser,
                               IN String    localServerName,
                               IN String    localServerProfile,
                               IN String    localServerType,
                               IN String    localDMHostName,
                               IN integer   localDMPort,
                               IN integer   localMinClusterMember,
                               IN String    localStartOptions,
                               IN String    localMonitorProgram,
                               IN integer   localSecondLevelMonitor,
                               IN String    LogLevel)

  Description : Validates the ArgList passed to the Agent Entry Point.

  Parameters :
  String    sEntryPointName          Entry Point Name
  String    localWASHome             Absolute Path to WAS install home
  String    localWASNode             Host on which WAS is configured
  String    localUser                Unix User for WAS instance
  String    localServerName          WAS Server Name
  String    localServerProfile       WAS Server Profile name (version 6x and above)
  String    localServerType          WAS Server Type
  String    localDMHostName          Deployment Manager HostName
  integer   localDMPort              SOAP Connector Port of Deployment manager
  integer   localMinClusterMember    Minimum number of cluster members running for cluster to be online
  String    localStartOptions        Start options for start script
  String    localMonitorProgram      Absolute path of external Monitor Prog
  integer   localSecondLevelMonitor  SecondLevel Monitor attrib. value
  String    LogLevel                 ACC Resource Log Level for messages

 Returns : boolean value in the set of {0,1} where
           0 false
           1 false

--------------------------------------------------------------------------------

=cut

sub ArgsValid 
{

  #----------------------------------------
  # set up local vars from passed in ones
  #----------------------------------------
  my($sEntryPointName,
     $localWASHome,
     $localWASNode,
     $localUser,
     $localServerName,
     $localServerProfile,
     $localServerType,
     $localDMHostName,
     $localDMPort,
     $localMinClusterMember,
     $localStartOptions,
     $localStopOptions,
     $localMonitorProgram,
     $localSecondLevelMonitor,
     $localLogLevel) = @_;

  my $sCryptString = $localStopOptions;
  if ($sCryptString =~ s/\s-password\s+\S+/ -password XXXXXX /g){
      # Nothing to do, continue
  }

  #-------------------------
  # Announce Entry
  #-------------------------
  my $sMyName = ( caller 0 )[3];
  &logTrace( 20001, "Subroutine <%s> called with:
      EntryPointName     [%s]
      WAS_HOME           [%s]
      WAS_NODE           [%s]
      User               [%s]
      ServerName         [%s]
      ServerProfile      [%s]
      ServerType         [%s]
      DMHostName         [%s]
      DMPort             [%s]
      MinClusterMember   [%s]
      StartOptions       [%s]
      StopOptions        [%s]
      MonitorProgram     [%s]
      SecondLevelMonitor [%s]",
      $sMyName,$sEntryPointName, $localWASHome, $localWASNode, $localUser, $localServerName, $localServerProfile, $localServerType, $localDMHostName,
      $localDMPort, $localMinClusterMember,  $localStartOptions, $sCryptString, $localMonitorProgram, $localSecondLevelMonitor );

  my $iReturnCode = 1;
  # Initialize Missing list and Args list
  my (@lsMissing, %hsArgList) = ();
  my @lsArgs = qw ( EntryPointName WAS_HOME WAS_NODE User ServerName
                    ServerProfile ServerType DMHostName DMPort MinClusterMember 
                    StartOptions StopOptions MonitorProgram SecondLevelMonitor );

  #----------
  # Assign
  #----------
  @hsArgList { @lsArgs  } = @_;

  #--------------------------------------------
  # Required attributes for all entry points.
  #--------------------------------------------
  my @lsReqdArgs = qw ( WAS_HOME WAS_NODE ServerName );

  #--------------------------------------------
  # If entrypoint is online/offline 
  #--------------------------------------------
  if ( $sEntryPointName =~ /^(on|off)line$/o )
  {
    push ( @lsReqdArgs , qw (ServerType User));
  }

  if ( ( defined $localServerType ) && ( $localServerType eq "Cluster" ) ) {
    push ( @lsReqdArgs , qw (DMHostName DMPort));
    if( $localDMPort < 0 ){
    &logError ( 20293, "<%s>: Invalid port [%s] specified.", $sMyName, $localDMPort );
    }
    if ( ! ( $localMinClusterMember =~ /^\d+$/ ) ) { 
    &logError ( 20294, "<%s>: Invalid Minimum Cluster Member count [%s] specified.", $sMyName, $localMinClusterMember );
    }
  }

  #------------------------
  # Check whats missing
  #------------------------
  if ( @lsMissing = grep { $hsArgList {$_} !~ /\S/o } @lsReqdArgs )
  {
    # Log Error
    &logError ( 20002, "<%s> The following mandatory fields were not set [%s],
    Total [%s]", $sMyName, join (',',@lsMissing), scalar @lsMissing);
    $iReturnCode = 0 ;
  }

  $iReturnCode || return $iReturnCode; # Return, if failed..

  #-------------------------------------------------------------
  # Check existence of WAS_HOME directory in online/offline EPs
  #-------------------------------------------------------------
  if ( $sEntryPointName eq "online" || $sEntryPointName eq "offline" )
  {
    if (! -d $localWASHome )
    {
      &logError(20003,"<%s>:WAS_HOME [%s] is not accessible.",
        $sMyName,$localWASHome);
      $iReturnCode = 0 ;
    }
  }

  #----------------------------------------------
  # Check if WAS version is 6.x and above
  # ServerProfile is mendatory
  #----------------------------------------------
  if ( !$localServerProfile ) {
    &logError(20333,"<%s>:ServerProfile is mandatory for WAS version 6.x and above.",$sMyName);
    $iReturnCode = 0 ;
  }

  #---------------------------------------------------------
  # Validate User and ServerType in online offline 
  #---------------------------------------------------------
  if ( $sEntryPointName eq "online" || $sEntryPointName eq "offline" )
  {
    #---------------------------------------------
    # Check whether the User is valid on system.
    #---------------------------------------------
    my $spwname = getpwnam($localUser);
    if ( !defined($spwname) )
    {
      &logError(20004, "<%s>:User name [%s] invalid.",$sMyName,$localUser);
      $iReturnCode = 0 ;
    }

    #---------------------------------------------------------------
    # Check whether ServerType is correct only for online/offline 
    #---------------------------------------------------------------
    if ( $sEntryPointName eq "online" || $sEntryPointName eq "offline" )
    {
      if (    $localServerType eq "NodeAgent" 
           || $localServerType eq "DeploymentManager" 
           || $localServerType eq "Cluster" 
           || $localServerType eq "ApplicationServer" )
      {
        #----------------------
        # Valid Server Type.
        #----------------------
        &logTrace(20005,"<%s>:ServerType set to [%s]",$sMyName,$localServerType);
      }
      else
      {
        #----------------------
        # Invalid Server Type.
        #----------------------
        &logError(20006,"<%s>:ServerType [%s] not supported.",
                  $sMyName,$localServerType);
        $iReturnCode = 0 ;
      }
    }
  }

  #----------------------------------------------------------
  # Validate MonitorProgram if passed during online
  #----------------------------------------------------------
  if ( $sEntryPointName eq "online" )
  {
    if ( $localMonitorProgram )
    {
      if ( ! -x $localMonitorProgram )
      {
        &logError(20007,"<%s> MonitorProgram [%s] not executable. Check file existence and permissons.",
                  $sMyName,$localMonitorProgram);
        $iReturnCode = 0 ;
      }
    }
  }

  if ( 0 == $iReturnCode )
  {
    #-----------------------------------
    # Some arguments validation failed.
    #-----------------------------------
    &logError ( 20008,"<%s> Some of the attributes validation failed.",$sMyName);
  }

  return $iReturnCode;
}

=pod

--------------------------------------------------------------------------------

 Function : bool Start ( IN String $sWASHome,
                         IN String $sWASNode,
                         IN String $sServerName,
                         IN String $sUser,
                         IN String $sStartOptions ,
                         IN String $sServerProfile ,
                         IN String $sResourceName ,
                         IN Int    $iSecondLevelMonitor ,
                         IN String $sResourceType,
                         IN String $sServerType,
                         IN String DMHostName,
                         IN integer DMPort,
                         IN integer MinClusterMember
                         IN Integer $iTimeOut );

 Description : Start subroutine for online entry point. It attempts to start
               the WebSphere instance if its not running already.

 Parameters :

 $sWASHome            Absolute Path to WAS install home
 $sWASNode            Host on which WAS is configured
 $sServerName         WAS Server Name
 $sUser               Unix User for WAS instance
 $sStartOptions       Start options for start script
 $sServerProfile      WAS Server Profile name (only 6x)
 $sResourceName       VCS ResourceName 
 $iSecondLevelMonitor SecondLevel Monitor attrib. value
 $sResourceType       VCS Resource Type
 $sServerType         WAS Server Type
 DMHostName           Deployment Manager Host name
 DMPort               SOAP Connector port of Deployment Manager
 MinClusterMember     Minimum number of cluster members running for cluster to be online
 $iTimeOut            Timeout value for this EP

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true

--------------------------------------------------------------------------------

=cut

sub Start 
{

  my ( $sWASHome,
       $sWASNode,
       $sServerName,
       $sUser,
       $sStartOptions ,
       $sServerProfile ,
       $sResourceName ,
       $iSecondLevelMonitor ,
       $sResourceType,
       $sServerType,
       $sDMHostName,
       $iDMPort,
       $iMinClusterMember,
       $iTimeOut    )   =  @_;

  #--------------------
  # Announce Entry
  #--------------------
  my $sMyName = ( caller 0 )[3];
  &logTrace( 20009, "Subroutine <%s> called with
    WASHome             [%s]
    WASNode             [%s]
    ServerName          [%s]
    User                [%s]
    StartOptions        [%s]
    ServerProfile       [%s]
    SecondLevelMonitor  [%s]
    ResourceType        [%s]
    ServerType          [%s] 
    DMHostName          [%s]
    DMPort              [%s]
    MinClusterMember    [%s]
    TimeOut             [%s]",
    $sMyName , $sWASHome , $sWASNode , $sServerName , $sUser ,
    $sStartOptions , $sServerProfile , $iSecondLevelMonitor,
    $sResourceType , $sServerType, $sDMHostName, $iDMPort, $iMinClusterMember, $iTimeOut );

  my $sWASBinDir  = "bin";
  my $sWASLogDir  = "logs";
  if ( $sServerProfile )
  {
    #-----------------------------------------
    # This is a version 6.x or above instance.
    #-----------------------------------------
    &logTrace(20010,"<%s> : Server version is 6.x and above",$sMyName);

    if ( $sServerProfile =~ m|^/| )
    {
      $sWASBinDir  = "$sServerProfile/bin";
      $sWASLogDir  = "$sServerProfile/logs";
      &logTrace(20011,"<%s> : Complete path to ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    }
    else
    {
      $sWASBinDir  = "profiles/$sServerProfile/bin";
      $sWASLogDir  = "profiles/$sServerProfile/logs";
      &logTrace(20012,"<%s> : Only name of ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    }
  }
  else
  {
    #-------------------------
    # This is a v5.x instance.
    #-------------------------
    &logTrace(20013,"<%s> : Server version is 5.x",$sMyName);

    # Note that:  $sWASBinDir  = "bin";
    # Note that:  $sWASLogDir  = "logs";
  }

=pod

  # Commenting this code because RunSecondLevelMonitor will 
  # handle this situation.

  #------------------------------------------------------------
  # Initialize the SLMCounter attribute to SecondLevelMonitor
  # We need to create the SLMCounter file in 
  # /tmp/.VRTS<ResourceType>/<<ResourceName>.slm
  # And set the counter value inside the file to 0.
  # First monitor should run SecondLevelMonitor hence we must
  # set the SLMCounter value inside the SLMFile to 1. So that
  # the very next monitor cycle will run SecondLevelMonitor.
  #------------------------------------------------------------

  my $iSLMCounter = 0;
  if ( $iSecondLevelMonitor )
  {
    $iSLMCounter = 1;
    &logTrace(20014,"Initializing SLMCounter to value [%s]",$iSLMCounter);
    WebSphere::SetSLMCounter($sResourceType,$sResourceName,$iSLMCounter);
  }
  else
  {
    &logTrace(20015,"Initializing SLMCounter to value [%s]",$iSLMCounter);
    WebSphere::SetSLMCounter($sResourceType,$sResourceName,$iSLMCounter);
  }
=cut

  #my $iTimeOut = 10;
  my $iReturnCode;

  #-----------------------------------------------
  # Call the actual start script provided by IBM.
  #-----------------------------------------------
  my $sStartScript = "";
  if ( $sServerProfile =~ m|^/| )
  {
    &logTrace(20016,"<%s> : Forming StartScript: Complete path to ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
  }
  else
  {
    &logTrace(20017,"<%s> : Forming StartScript: Only name of ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    $sWASBinDir = "$sWASHome/$sWASBinDir";
  }

  if ( $sServerType eq "ApplicationServer" )
  {
    $sStartScript = "$sWASBinDir/".WebSphere::WAS_START_SERVER;
  }
  elsif ( $sServerType eq "NodeAgent" )
  {
    $sStartScript = "$sWASBinDir/".WebSphere::WAS_START_NODE;
  }
  elsif ( $sServerType eq "DeploymentManager" )
  {
    $sStartScript = "$sWASBinDir/".WebSphere::WAS_START_MANAGER;
  }
  elsif ( $sServerType eq "Cluster" )
  {
    $sStartScript = "$sWASBinDir/".WebSphere::WAS_WSADMIN;
  }

  if (! -x $sStartScript )
  {
    &logError(20018,"<%s>:Start script [%s] is not executable. Check file existence and permissons.",$sMyName,$sStartScript);
    return 0;
  }

  #-------------------------------------------------------------------
  # Handle embedded spaces and (IBM supported) special characters 
  # in WAS Server Name. Need to take care of spaces and tabs when
  # executing a command with those characters.
  #-------------------------------------------------------------------
  my $sRegexWASName = GetServerNameRegex($sServerName);
  $sRegexWASName =~ s/ /\\ /g;
  $sRegexWASName =~ s/\t/\\\t/g;
  $sStartOptions = $sRegexWASName." ".$sStartOptions if ($sServerType eq "ApplicationServer");

  #-------------------------------------------------------------------
  # $sStartOptions argument forms wsadmin commands to create cluster 
  # object and execute cluster operations on these cluster objects
  # Command to Start cluster
  #wsadmin.sh -lang jython -conntype SOAP -host $sDMHostName -port $iDMPort -c "clusterName='$sServerName'; clusterObj = AdminControl.completeObjectName('type=Cluster,name='+clusterName+',*'); AdminControl.invoke( clusterObj, 'start');" 
  #-------------------------------------------------------------------
  if ( $sServerType eq "Cluster" ){
    $sStartOptions = "-lang jython -conntype SOAP -host ".$sDMHostName." -port ".$iDMPort." -c \"clusterName=\'".$sServerName."\'; clusterObj = AdminControl.completeObjectName('type=Cluster,name='+clusterName+',*'); AdminControl.invoke( clusterObj, 'start');\""; 
  }

  #------------------------------------------
  # Determine the expiration time of the EP.
  #------------------------------------------
  my ($sEntryPointName) = reverse $0 =~ m@([^/]+)@g;

  #-------------------------
  # Execute the start script
  #-------------------------
  my $sEnvFile = "";
  my $iSourceEnvFlag = 0;
  my $iExitCode = 0;
  my @lsCommandOut = ();
  my $bTimedOut;
  my $iEntryPointStartTime = $^T;
  my $iTillWhen = ( $^T + $iTimeOut ) - WebSphere::EPOP;
  $iTimeOut = $iTimeOut - (time - $iEntryPointStartTime) - 3 ;

  #--------------------------------------------------------------------------
  # Form the WAS Log File whether the output of the CLI will be redirected.
  # Remove the WAS Log File, just in case it exists and somehow the
  # environment that we run within has somehow set the shell to run
  # with noclobber.  Note that we do not check the success of the unlink,
  # we just want to remove it if it exists...
  #--------------------------------------------------------------------------

  my $sEntryPoint   =   $ENV{'VCS_LOG_SCRIPT_NAME'};
  $sEntryPoint      =~  s/.*\///g;
  my $sWASLogFile   =   "/tmp/$sResourceName.$sEntryPoint.$$";
  unlink ($sWASLogFile);
  if (SystemD::DetectSystemd() && $sServerType ne "Cluster") {
  #runServicefile
    $iReturnCode = SystemD::RunSystemdWithEnv($sUser, $sStartScript, $sStartOptions, $sEnvFile, $iTimeOut, undef);
    if($iReturnCode) {
      &logTrace( 20394,"<%s>: RunSystemdServiceName call executed successfully",$sMyName);
    }
    else {
      &logError( 20395,"<%s>: RunSystemdServiceName call failed to execute",$sMyName);
    }
  }
  else {
  $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut($sEnvFile,
                                                  $iSourceEnvFlag,
                                                  $sStartScript,
                                                  $sStartOptions,
                                                  \$iExitCode,
                                                  $sUser,
                                                  \@lsCommandOut,
                                                  \$bTimedOut,
                                                  $iTimeOut,
                                                  $sWASLogFile,
                                                  "$sWASBinDir");
  }

  if ( $iReturnCode ){
    #----------------------
    # Command got executed.
    #----------------------
    if ( $iExitCode == 0 ){
      &logTrace( 20021, "<%s>: Command [%s] executed successfully.", $sMyName, $sStartScript );
    }
    elsif ( $iExitCode == 1 ){
      if( $sServerType eq "Cluster" ){
        &logError( 20370, "<%s>: Specified profile [%s] cannot be found or profile not linked correctly.", $sMyName, $sServerProfile );
      }
      elsif( $sServerType eq "NodeAgent" || $sServerType eq "ApplicationServer" || $sServerType eq "DeploymentManager" ){
        &logError (20020,"<%s>:[%s] with args [%s] returned exit code [%s]",
               $sMyName,$sStartScript,$sStartOptions,$iExitCode);
      }
      return 0;
    }
    elsif ( $iExitCode == 103 ){
      if( $sServerType eq "Cluster" ){
        &logError( 20371, "<%s>: Wrong Server Profile [%s] specified or Deployment Manager is not ruinnig or some error occured while executing [%s].", $sMyName, $sServerProfile,$sStartScript );
      }
      elsif( $sServerType eq "NodeAgent" || $sServerType eq "ApplicationServer" || $sServerType eq "DeploymentManager" ){
      &logError ( 20372, "<%s>:[%s] with args [%s] returned exit code [%s]",
                 $sMyName,$sStartScript,$sStartOptions,$iExitCode);
      }
      return 0;
    }
    else{
      &logError( 20373, "<%s>: Unknown error occured. Command [%s] with args [%s] returned exit code [%s]", $sMyName, $sStartScript,$sStartOptions,$iExitCode );
      return 0;
    }
  }
  else
  {
    if ( $bTimedOut )
    {
      #-------------------
      # Command timed out.
      #-------------------
      &logError (20022,"<%s>:[%s] did not complete within [%s] seconds.",
                 $sMyName,$sStartScript,$iTimeOut);
    }
    else
    {
      #-------------------
      # Command failed.
      #-------------------
      &logError (20023,"<%s>:[%s] failed.",
                 $sMyName,$sStartScript);
    }
    return 0;
  }

  #-----------------------------------------------------------------
  # Now delay the EP till the resource becomes completely online.
  #-----------------------------------------------------------------

  $iReturnCode = DelayEntryPoint( $sWASHome,
                                  $sWASNode,
                                  $sServerName,
                                  $sUser,
                                  $sServerProfile,
                                  $sServerType,
                                  $sDMHostName,
                                  $iDMPort,
                                  $iMinClusterMember,
                                  $iTillWhen );
  &logTrace (20024, "<%s>: DelayEntryPoint ReturnCode [%d] .", $sMyName, $iReturnCode);
  unlink ($sWASLogFile);
  return $iReturnCode;
}

=pod

--------------------------------------------------------------------------------

 Function : bool Stop  ( IN String $sWASHome,
                         IN String $sWASNode,
                         IN String $sServerName,
                         IN String $sUser,
                         IN String $sStopOptions ,
                         IN String $sServerProfile ,
                         IN String $sResourceType,
                         IN String $sServerType,
                         IN String DMHostName,
                         IN integer DMPort,
                         IN integer MinClusterMember,
                          IN String $iTimeOut );

 Description : Stop subroutine for offline entry point. It attempts to stop
               the WebSphere instance if its running.

 Parameters :

 $sWASHome          Absolute Path to WAS install home
 $sWASNode          Host on which WAS is configured
 $sServerName       WAS Server Name
 $sUser             Unix User for WAS instance
 $sStopOptions      Stop options for stop script
 $sServerType       WAS Server Type
 $sResourceType     VCS Resource Type
 $sServerType       WAS Server Type
 DMHostName         Deployment Manager Host name
 DMPort             SOAP Connector port of Deployment Manager
 MinClusterMember   Minimum number of cluster members running for cluster to be online
 $iTimeOut          Timeout value for this EP

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true

--------------------------------------------------------------------------------

=cut

sub Stop 
{

  my ( $sWASHome,
       $sWASNode,
       $sServerName,
       $sUser,
       $sStopOptions ,
       $sServerProfile ,
       $sResourceType , 
       $sServerType,
       $sDMHostName,
       $iDMPort,
       $iMinClusterMember,
       $iTimeOut,
       $sAdminUserName,
       $sAdminPassword )   =  @_;

  #--------------------
  # Announce Entry
  #--------------------
  my $sCryptString = $sStopOptions;
  if ($sCryptString =~ s/\s-password\s+\S+/ -password XXXXXX /g){
      # Nothing to do, continue
  }
       
  my $sMyName = ( caller 0 )[3];
  &logTrace( 20025, "Subroutine <%s> called with
    WASHome             [%s]
    WASNode             [%s]
    ServerName          [%s]
    User                [%s]
    StopOptions         [%s]
    ServerProfile       [%s]
    ResourceType        [%s]
    ServerType          [%s] 
    DMHostName          [%s]
    DMPort              [%s]
    MinClusterMember    [%s]
    TimeOut             [%s]
    AdminUserName       [%s]",
    $sMyName , $sWASHome , $sWASNode , $sServerName , $sUser ,
    $sCryptString, $sServerProfile , $sResourceType,
    $sServerType, $sDMHostName, $iDMPort, $iMinClusterMember, $iTimeOut, $sAdminUserName );

  my $sWASBinDir  = "bin";
  my $sWASLogDir  = "logs";
  my ($sCmdDecrypt, $sDecryptedAdminPassword, $iExitCode) = ();

  #-------------------------
  # Get decrypted password..
  #-------------------------
  if ( $sAdminPassword =~ /\S+/ ) {
    #-----------
    # Validate..
    #-----------
    if ( ! -x $sCmdVcsDecrypt ) {
      &logError ( 20337,
      "<%s> Command [%s] is not executable", $sMyName, $sCmdVcsDecrypt );
      return 0; # Failure..
    }

    $sCmdDecrypt = $sCmdVcsDecrypt;                       # Get global..
    ( $sDecryptedAdminPassword ) = grep {                 # Get decrypted..
                          s/([^\w-])/\\$1/g || 1          # Escape non-words..
                        } readpipe ( "$sCmdDecrypt $sAdminPassword" );

    #--------------------
    # Collect exit code..
    #--------------------
    if ( $iExitCode = $? >> 8 ) {
      &logError ( 20338, "<%s> Failed to decrypt crypted string [%s]",$sMyName, $sAdminPassword);
      return 0; # Failure..
    }
  }



  if ( $sServerProfile )
  {
    #-----------------------------------------
    # This is a version 6.x or above instance.
    #-----------------------------------------
    &logTrace(20026,"<%s> : Server version is 6.x and above",$sMyName);

    if ( $sServerProfile =~ m|^/| )
    {
      $sWASBinDir  = "$sServerProfile/bin";
      $sWASLogDir  = "$sServerProfile/logs";
      &logTrace(20027,"<%s> : Complete path to ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    }
    else
    {
      $sWASBinDir  = "profiles/$sServerProfile/bin";
      $sWASLogDir  = "profiles/$sServerProfile/logs";
      &logTrace(20028,"<%s> : Only name of ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    }
  }
  else
  {
    #-------------------------
    # This is a v5.x instance.
    #-------------------------
    &logTrace(20029,"<%s> : Server version is 5.x",$sMyName);

    # Note that:  $sWASBinDir  = "bin";
    # Note that:  $sWASLogDir  = "logs";
  }

  #my $iTimeOut = 10;
  my $iReturnCode;

  #-----------------------------------------------
  # Call the actual stop script provided by IBM.
  #-----------------------------------------------
  my $sStopScript = "";

  if ( $sServerProfile =~ m|^/| )
  {
    &logTrace(20030,"<%s> : Forming StopScript: Complete path to ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
  }
  else
  {
    &logTrace(20031,"<%s> : Forming StopScript: Only name of ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    $sWASBinDir = "$sWASHome/$sWASBinDir";
  }


  if ( $sServerType eq "ApplicationServer" )
  {
    $sStopScript = "$sWASBinDir/".WebSphere::WAS_STOP_SERVER;
  }
  elsif ( $sServerType eq "NodeAgent" )
  {
    $sStopScript = "$sWASBinDir/".WebSphere::WAS_STOP_NODE;
  }
  elsif ( $sServerType eq "DeploymentManager" )
  {
    $sStopScript = "$sWASBinDir/".WebSphere::WAS_STOP_MANAGER;
  }
  elsif ( $sServerType eq "Cluster" )
  {
    $sStopScript = "$sWASBinDir/".WebSphere::WAS_WSADMIN;
  }

  if (! -x $sStopScript )
  {
    &logError(20032,"<%s>:Stop script [%s] is not executable. Check file existence and permissons.",$sMyName,$sStopScript);
    return 0;
  }

  #-------------------------------------------------------------------
  # Handle embedded spaces and (IBM supported) special characters 
  # in WAS Server Name. Need to take care of spaces and tabs when
  # executing a command with those characters.
  #-------------------------------------------------------------------
  my $sRegexWASName =  GetServerNameRegex($sServerName);
  $sRegexWASName    =~ s/ /\\ /g;
  $sRegexWASName    =~ s/\t/\\\t/g;
  $sStopOptions     =  $sRegexWASName." ".$sStopOptions if ($sServerType eq "ApplicationServer");

  #-------------------------------------------------------------------
  # $sStopOptions argument forms wsadmin commands to create cluster 
  # object and execute cluster operations on these cluster objects
  # Command to stop cluster
  # wsadmin.sh -lang jython -conntype SOAP -host $sDMHostName -port $iDMPort -c "clusterName='$sServerName'; clusterObj = AdminControl.completeObjectName('type=Cluster,name='+clusterName+',*'); AdminControl.invoke( clusterObj, 'stop');"
  #-------------------------------------------------------------------
  if ( $sServerType eq "Cluster" ){
    $sStopOptions = "-lang jython -conntype SOAP -host ".$sDMHostName." -port ".$iDMPort." -c \"clusterName=\'".$sServerName."\'; clusterObj = AdminControl.completeObjectName('type=Cluster,name='+clusterName+',*'); AdminControl.invoke( clusterObj, 'stop');\" "; 
  }

#  if($sAdminUserName =~ /\S+/) {
#      if($sStopOptions !~ /-user/) {
#         $sStopOptions = $sStopOptions . "  -username " . $sAdminUserName;
#      } else { 
#         &logInfo ( 20339, "<%s>: [-username] already provided in StopOptions, hence not using AdminUserName attribute value [%s]", $sMyName, $sAdminUserName);
#      }
#  }
#
#  if($sAdminPassword =~ /\S+/) {
#     if ($sStopOptions !~ /-password/){
#         $sStopOptions = $sStopOptions . "  -password " . $sDecryptedAdminPassword;
#     } else {
#        &logInfo ( 20340, "<%s>: [-password] already provided in StopOptions, hence not using AdminPassword attribute value [%s]", $sMyName, $sAdminPassword);
#     }
#  }
  #------------------------------------------
  # Determine the expiration time of the EP.
  #------------------------------------------
  my ($sEntryPointName) = reverse $0 =~ m@([^/]+)@g;

  #----------------------------------------
  # Remove the SLM Counter file in /tmp
  #----------------------------------------
  my $sResourceName =   $ENV{'VCS_LOG_RESOURCE_NAME'};
  &logTrace(20034,"<%s>: Unlinking [%s]",$sMyName,"/tmp/.VRTS$sResourceType/$sResourceName.slm");
  unlink ("/tmp/.VRTS$sResourceType/$sResourceName.slm");

  #-------------------------
  # Execute the stop script
  #-------------------------
  my $sEnvFile = "";
  my $iSourceEnvFlag = 0;
  $iExitCode = 0;
  my @lsCommandOut = ();
  my $bTimedOut;
  my $iEntryPointStartTime = $^T;
  $iTimeOut = $iTimeOut - (time - $iEntryPointStartTime) - 3 ;
  my $iTillWhen = ( $^T + $iTimeOut ) - WebSphere::EPOP;

  if ( $sServerType eq "Cluster" ){
  #-----------------------------------------
  # Retrieve resource state on all systems.
  #-----------------------------------------
    $sResourceName = $ENV { VCS_LOG_RESOURCE_NAME };
    my ( @sSystemList, @lsOut, @lsHaresOut, %hSysState, $sSystem ) = ();
    my $sCommand = $sCmdHares;
    my $sCommandArgs = "-state $sResourceName";
    &logTrace ( 20288,"<%s>: Running command [%s] with arguments [%s].", $sMyName, $sCommand, $sCommandArgs );
    @lsHaresOut = grep { (s/\n// || 1) } readpipe ( "$sCommand $sCommandArgs" );

  #-----------------------------------------
  # Check if command executed successfully..
  #-----------------------------------------
    if ( $iExitCode = $? >> 8 ) {
      &logError ( 20287,"<%s>: Cmd [%s] with args [%s] failed with exit code [%s].", $sMyName, $sCommand, $sCommandArgs, $iExitCode );
    }
    else {
      shift @lsHaresOut;
      foreach my $sOut (@lsHaresOut) {
        my @lsOut = split( '\s+', $sOut);
        $hSysState{ $lsOut[2] } = $lsOut[3];
        push @sSystemList, $lsOut[2];
      }
      &logTrace ( 20286,"<%s>: SystemList is [%s].", $sMyName, join(',',@sSystemList) );
    }

    my $sCurrentHost = "";
    if ( GetNodeId( \$sCurrentHost ) ){
      &logError ( 20274, "<%s>: Couldn't retrieve node name.", $sMyName );
      return 0;
    }
    else{
      &logTrace( 20282,"<%s>: Current host is [%s]", $sMyName, $sCurrentHost );
    }

  #---------------------------------------------------------------------------
  # If reources on other system are online make them offline.
  # This is required because if resource goes offline unexpectedly from 
  # steady online state, resource on other nodes fault and clean is called.
  #--------------------------------------------------------------------------
    $sCommand = $sCmdHares;
    foreach $sSystem ( keys %hSysState ){
      if ( $sSystem ne $sCurrentHost ){
        $sCommandArgs = "-offline $sResourceName -sys $sSystem";
        if ( $hSysState{ $sSystem } eq "ONLINE" ){
          &logInfo ( 20285, "<%s>: Resource [%s] is online on system [%s]. Running command [%s] with args [%s].", $sMyName, $sResourceName, $sSystem, $sCommand, $sCommandArgs );
          @lsOut = grep { (s/\n// || 1) } readpipe ( "$sCommand $sCommandArgs" );
          if ( $iExitCode = $? >> 8 ) {
            &logError ( 20284,"<%s>: Cmd [%s] with args [%s] failed with exit code [%s]. Command output is [%s]", $sMyName, $sCommand, $sCommandArgs, $iExitCode, join(' ', @lsOut) );
          }
        }
      }
    }
  }

  #--------------------------------------------------------------------------
  # Form the WAS Log File whether the output of the CLI will be redirected.
  # Remove the WAS Log File, just in case it exists and somehow the
  # environment that we run within has somehow set the shell to run
  # with noclobber.  Note that we do not check the success of the unlink,
  # we just want to remove it if it exists...
  #--------------------------------------------------------------------------
  my $sEntryPoint   =   $ENV{'VCS_LOG_SCRIPT_NAME'};
  $sEntryPoint      =~  s/.*\///g;
  my $sWASLogFile   =   "/tmp/$sResourceName.$sEntryPoint.$$";
  unlink ($sWASLogFile);

  $sCryptString  = "XXXXX";
  if($sStopOptions =~ /\s+-password\s+(\S+)/){
     $sCryptString = $1;
  }

  $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut($sEnvFile,
                                                    $iSourceEnvFlag,
                                                    $sStopScript,
                                                    $sStopOptions,
                                                    \$iExitCode,
                                                    $sUser,
                                                    \@lsCommandOut,
                                                    \$bTimedOut,
                                                    $iTimeOut,
                                                    $sWASLogFile,
                                                    "$sWASBinDir",
                                                    $sCryptString);

  if ( $iReturnCode )
  {
    #----------------------
    # Command got executed.
    #----------------------
    if ( $iExitCode == 1 )
    {
      #----------------
      # Command failed.
      #----------------
      if( $sServerType eq "Cluster" ){
        &logError( 20374, "<%s>: Specified profile [%s] cannot be found or profile not linked correctly.", $sMyName, $sServerProfile );
      }
      elsif( $sServerType eq "NodeAgent" || $sServerType eq "ApplicationServer" || $sServerType eq "DeploymentManager" ){
        &logError (20035,"<%s>:[%s] returned exit code [%s]",
            $sMyName,$sStopScript,$iExitCode);
      }
      return 0;
    }
    elsif ( $iExitCode == 0 ){
      #--------------------------------
      # Command executed successfully.
      #--------------------------------
        &logTrace( 20036, "<%s>: Command [%s] executed successfully.", $sMyName, $sStopScript );
    }
    elsif ( $iExitCode == 103 ){
      if( $sServerType eq "Cluster" ){
        &logError( 20375, "<%s>: Wrong Server Profile [%s] specified or Deployment Manager is not runnig or some error occured while executing [%s].", $sMyName, $sServerProfile,$sStopScript );
      }
      elsif( $sServerType eq "NodeAgent" || $sServerType eq "ApplicationServer" || $sServerType eq "DeploymentManager" ){
        &logError ( 20376, "<%s>:[%s] with args [%s] returned exit code [%s]",
            $sMyName,$sStopScript,$sStopOptions,$iExitCode);
      }
      return 0;
    }
    else{
        &logError( 20377, "<%s>: Unknown error occured. Command [%s] with args [%s] returned exit code [%s]", $sMyName,$sStopScript,$sStopOptions,$iExitCode );
      return 0;
    }
  }
  else
  {
    if ( $bTimedOut )
    {
      #-------------------
      # Command timed out.
      #-------------------
      &logError (20037,"<%s>:[%s] did not complete within [%s] seconds.",
          $sMyName,$sStopScript,$iTimeOut);
    }
    else
    {
      #-------------------
      # Command failed.
      #-------------------
      &logError (20038,"<%s>:[%s] failed.",
          $sMyName,$sStopScript);
    }
    return 0;
  }

  #-----------------------------------------------------------------
  # Now delay the EP till the resource becomes completely offline.
  #-----------------------------------------------------------------

  $iReturnCode = DelayEntryPoint( $sWASHome,
                                  $sWASNode,
                                  $sServerName,
                                  $sUser,
                                  $sServerProfile,
                                  $sServerType,
                                  $sDMHostName,
                                  $iDMPort,
                                  $iMinClusterMember,
                                  $iTillWhen );
  &logTrace (20039, "<%s>: DelayEntryPoint ReturnCode [%d] .", $sMyName, $iReturnCode);
  unlink ($sWASLogFile);
  return $iReturnCode;
}


=pod

--------------------------------------------------------------------------------

 Function : Int GetResourceState ( IN String WASHome ,
                                   IN String ServerName ,
                                   IN String WASNode ,
                                   IN String ServerProfile ,
                                   IN String ServerType,
                                   IN String DMHostName,
                                   IN Int    DMPort,
                                   IN Int    MinClusterMember,
                                   IN Int    SecondLevelMonitor ,
                                   IN String MonitorProgram ,
                                   IN Int    SLMCounter ,
                                   IN Int    RelevantTimeout ,
                                   IN String ResourceName ,
                                   IN String User ,
                                   IN Int    LevelOneMonitor ,
                                   IN Int    LevelTwoMonitor )

 Description : Monitoring subroutine to determine state of WAS.

 Parameters :
 String WASHome                WAS installation home
 String ServerName             WAS Server name
 String WASNode                WAS Node
 String ServerProfile          WAS Server Profile name (only 6x)
 String ServerType             WAS resource type 
 String localDMHostName        Deployment Manager HostName
 integer localDMPort           SOAP Connector Port of Deployment manager
 integer MinClusterMember      Minimum number of cluster members running for cluster to be online
 Int    SecondLevelMonitor     Second Level Monitor Value
 String MonitorProgram         External Monitor Program
 Int    RelevantTimeout        Timeout value for this EP
 String ResourceName           Resource Name
 String User                   Unix User Name

 Returns : Integer value as defined below:

  99  UNKNOWN                                  VCSDef_MonitorExit::UNKNOWN
 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  PARTIAL_ONLINE                           VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100         VCSDef_MonitorExit::ONLINE

--------------------------------------------------------------------------------

=cut

sub GetResourceState
{
  my ( $sWASHome,
       $sServerName,
       $sWASNode,
       $sServerProfile,
       $sServerType,
       $sDMHostName,
       $iDMPort,
       $iMinClusterMember,
       $iSecondLevelMonitor,
       $sMonitorProgram,
       $sResourceType,
       $sResourceName,
       $sUser,
       $iTimeOut,
       $level_one,
       $level_two ) = @_;

  #------------------
  # Announce Entry.
  #------------------
  my ($sMyName ) = (caller 0)[3];
  &logTrace( 20040, "Subroutine <%s> called with args
    WASHome              [%s]
    ServerName           [%s]
    WASNode              [%s]
    ServerProfile        [%s]
    ServerType           [%s]
    DMHostName           [%s]
    DMPort               [%s]
    MinClusterMember     [%s]
    SecondLevelMonitor   [%s]
    MonitorProgram       [%s]
    ResourceType         [%s]
    ResourceName         [%s]
    User                 [%s]
    Timeout              [%s]
    level_one            [%s]
    level_two            [%s]",
    $sMyName, $sWASHome, $sServerName, $sWASNode, $sServerProfile, $sServerType, $sDMHostName, $iDMPort, $iMinClusterMember,
    $iSecondLevelMonitor, $sMonitorProgram, $sResourceType , $sResourceName, $sUser,$iTimeOut, $level_one, $level_two );

  my $sWASBinDir  = "bin";
  my $sWASLogDir  = "logs";
  my $iRelevantTimeout = $iTimeOut;
  my $iEntryPointStartTime = $^T;
  if ( $sServerProfile )
  {
    #-----------------------------------------
    # This is a version 6.x or above instance.
    #-----------------------------------------
    &logTrace(20041,"<%s> : Server version is 6.x and above",$sMyName);

    if ( $sServerProfile =~ m|^/| )
    {
      $sWASBinDir  = "$sServerProfile/bin";
      $sWASLogDir  = "$sServerProfile/logs";
      &logTrace(20042,"<%s> : Complete path to ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    }
    else
    {
      $sWASBinDir  = "profiles/$sServerProfile/bin";
      $sWASLogDir  = "profiles/$sServerProfile/logs";
      &logTrace(20043,"<%s> : Only name of ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    }
  }
  else
  {
    #-------------------------
    # This is a v5.x instance.
    #-------------------------
    &logTrace(20044,"<%s> : Server version is 5.x",$sMyName);
    # Note that:  $sWASBinDir  = "bin";
    # Note that:  $sWASLogDir  = "logs";
  }

  #my $iTimeOut                = 10;
  my $iReturnCode;
  if ( 1 == $level_one ) {
    $iReturnCode = WebSphere::GetWASProcState($sWASHome,
                                    $sWASNode,
                                    $sServerName,
                                    $sUser,
                                    $sServerProfile,
                                    $sServerType,
                                     $sDMHostName,
                                    $iDMPort,
                                    $iMinClusterMember,
                                    $iTimeOut);

    if ( 0 == $iReturnCode || 2 == $iReturnCode )
    {
      if( $sServerType eq "Cluster" ){
        &logTrace( 20378,"WAS cluster is stopped.");
      }
      elsif( $sServerType eq "NodeAgent" || $sServerType eq "ApplicationServer" || $sServerType eq "DeploymentManager" ){
      #-----------------------------
      # 0 - WAS pid is not running.
      # 2 - WAS pid a zombie.
      #-----------------------------
      &logTrace(20045,"WAS process is not running.");
      }
      return VCSDef_MonitorExit::OFFLINE;
    }

    if ( 3 == $iReturnCode )
    {
      &logTrace(20046,"WAS process/cluster status could not be determined.");
      return VCSDef_MonitorExit::UNKNOWN;
    }
  }

  #--------------------------------------------------
  # We reached here because GetWASProcState returned 
  # the resource to be online.
  # Call SecondLevelMonitor 
  # Execute External MonitorProgram if specified.
  #--------------------------------------------------

  if ( $sServerProfile =~ m|^/| )
  {
    &logTrace(20047,"<%s> : Complete path to ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
  }
  else
  {
    &logTrace(20048,"<%s> : Only name of ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    $sWASBinDir = "$sWASHome/$sWASBinDir";
  }

  #---------------------------------------
  # Do Second level monitoring if enabled.
  #---------------------------------------
  if ( -300 == $level_two ) {
     $level_two = RunSecondLevelMonitor ( $iSecondLevelMonitor );  
     &logTrace ( 20307, "<%s> Traditional second level monitoring, SLM flag for this monitor set to [%s] ", $sMyName, $level_two  );
  }

    if ( $level_two >= 1 )
    {
      my $iCurrentTime         = time;
      $iTimeOut = $iRelevantTimeout -($iCurrentTime - $iEntryPointStartTime) - 3;
#-------------------------
# Call SecondLevelMonitor
#-------------------------
      $iReturnCode = SecondLevelMonitor($sWASHome,
                                        $sWASNode,
                                        $sServerName,
                                        $sServerProfile,
                                        $sServerType,
                                        $sDMHostName,
                                        $iDMPort,
                                        $iMinClusterMember,
                                        $sWASBinDir,
                                        $sUser,
                                        $iTimeOut);
      if (   VCSDef_MonitorExit::UNKNOWN == $iReturnCode 
          || VCSDef_MonitorExit::OFFLINE == $iReturnCode )
      {
        return $iReturnCode;
      }
    }
  #---------------------------------------
  # if MonitorProgram is set , execute it.
  #---------------------------------------
  if ( $sMonitorProgram )
  {
    #------------------------------------------
    # Determine the expiration time of the EP.
    #------------------------------------------
    my ($sEntryPointName) = reverse $0 =~ m@([^/]+)@g;


    my $iCurrentTime         = time;
    $iTimeOut = $iRelevantTimeout -($iCurrentTime - $iEntryPointStartTime) - 3;
    my $sEnvFile = "";
    my $iSourceEnvFlag = 0;
    my $iExitCode;
    my @lsCommandOut;
    my $bTimedOut;
    my $sEPTimeout;
    $sEPTimeout = "MonitorTimeout";

    $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut($sEnvFile,
                                                    $iSourceEnvFlag,
                                                    $sMonitorProgram,
                                                    "",
                                                    \$iExitCode,
                                                    $sUser,
                                                    \@lsCommandOut,
                                                    \$bTimedOut,
                                                    $iTimeOut,
                                                    "",
                                                    "$sWASBinDir");
    if ( $iReturnCode )
    {
      #----------------------
      # Command got executed.
      #----------------------
      if ( 100 == $iExitCode || 1 == $iExitCode )
      {
        #------------------------------------------------------
        # Monitor Program returned WAS instance to be offline.
        #------------------------------------------------------
        &logTrace (20054,"<%s>:[%s] returned exit code [%s]",
                   $sMyName,$sMonitorProgram,$iExitCode);
        &logError (20055,"<%s>:[%s] reports resource state PARTIAL_ONLINE",
                   $sMyName,$sMonitorProgram);
        return VCSDef_MonitorExit::PARTIAL_ONLINE;
      }
      elsif ( 110 == $iExitCode || 0 == $iExitCode )
      {
        #------------------------------------------------------
        # Monitor Program returned WAS instance to be online.
        #------------------------------------------------------
        &logTrace ( 20369, "<%s>:[%s] return exit code[%s].",
                  $sMyName, $sMonitorProgram, $iExitCode);
        return VCSDef_MonitorExit::ONLINE;
      }
      else
      {
        #-------------------------
        # WAS status is Unknown 
        #-------------------------
        &logError (20056,"<%s>:[%s] reports resource state UNKNOWN",
                   $sMyName,$sMonitorProgram);
        return VCSDef_MonitorExit::UNKNOWN;
      }
    }
    else
    {
      if ( $bTimedOut )
      {
        #-------------------
        # Command timed out.
        #-------------------
        &logError (20057,"<%s>:[%s] did not complete within [%s] seconds.",
                   $sMyName,$sMonitorProgram,$iTimeOut);
      }
      else
      {
        #-------------------
        # Command failed.
        #-------------------
        &logError (20058,"<%s>:[%s] failed.",
                   $sMyName,$sMonitorProgram);
      }
      #-------------------------
      # WAS status is Unknown 
      #-------------------------
      return VCSDef_MonitorExit::UNKNOWN;
    }
  }
  return VCSDef_MonitorExit::ONLINE;
}

=pod

--------------------------------------------------------------------------------

 Function : Int GetWASProcState ( IN String WASHome ,
                                  IN String WASNode ,
                                  IN String ServerName ,
                                  IN String User ,
                                  IN String ServerProfile,
                                  IN String ServerType,
                                  IN String DMHostName,
                                  IN integer DMPort,
                                  IN integer MinClusterMember,
                                  IN Int     RelevantTimeout );

 Description : Monitoring subroutine to determine state of WAS.
 Determine if WAS processes are running.
 This routine first checks the pid file for the WAS instance.
 If process from pid file is not running, it will check whether
 processes are running with different pid. IE the pid file is stale.
 returns 0 if no processes are running.
 returns 1 if processes for this WAS instance are running.

 Parameters :
 String WASHome                WAS installation home
 String WASNode                WAS Node
 String User                   Unix User Name
 String ServerName             WAS Server name
 String ServerProfile          WAS Server Profile name (only 6x)
 String ServerType             WAS Server type
 String DMHostName             Deployment Manager Host name
 integer DMPort                SOAP Connector port of Deployment Manager
 integer MinClusterMember      Minimum number of cluster members running for cluster to be online
 Int     RelevantTimeout       Timeout value for this EP

 Returns : Integer value as defined below:

 0  No processes running for this WAS instance.
 1  Processes found to be running for this WAS instance.

--------------------------------------------------------------------------------

=cut


sub GetWASProcState 
{

  my ( $sWASHome, 
       $sWASNode, 
       $sServerName, 
       $sUser,
       $sServerProfile,
       $sServerType,
       $sDMHostName,
       $iDMPort,
       $iMinClusterMember,
       $iTimeOut ) = @_;

  #--------------------
  # Announce Entry.
  #--------------------
  my ($sMyName ) = (caller 0)[3];
  &logTrace( 20059, "Subroutine: <%s> called with args:
    WASHome         [%s]
    WASNode         [%s]
    ServerName      [%s]
    User            [%s]
    ServerProfile   [%s]
    ServerType      [%s]
    DMHostName      [%s]
    DMPort          [%s]
    MinClusterMember[%s]
    TimeOut         [%s]",
    $sMyName, $sWASHome, $sWASNode, $sServerName, $sUser, $sServerProfile, 
    $sServerType, $sDMHostName, $iDMPort, $iMinClusterMember, $iTimeOut );

  my $sWASBinDir  = "bin";
  my $sWASLogDir  = "logs";
  my $iRelevantTimeout = $iTimeOut;
  my $iEntryPointStartTime = $^T;
  my $iReturnCode = 1;                        # Ser return code optimistically to success.
  if ( $sServerProfile )
  {
    #-----------------------------------------
    # This is a version 6.x or above instance.
    #-----------------------------------------
    &logTrace(20060,"<%s> : Server version is 6.x and above",$sMyName);

    if ( $sServerProfile =~ m|^/| )
    {
      $sWASBinDir  = "$sServerProfile/bin";
      $sWASLogDir  = "$sServerProfile/logs";
      &logTrace(20061,"<%s> : Complete path to ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    }
    else
    {
      $sWASBinDir  = "profiles/$sServerProfile/bin";
      $sWASLogDir  = "profiles/$sServerProfile/logs";
      &logTrace(20062,"<%s> : Only name of ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    }
  }
  else
  {
    #-------------------------
    # This is a v5.x instance.
    #-------------------------
    &logTrace(20063,"<%s> : Server version is 5.x",$sMyName);

    # Note that:  $sWASBinDir  = "bin";
    # Note that:  $sWASLogDir  = "logs";
  }

  #-----------------------------------------------------------
  # First try to read the pid from pid file and check whether 
  # its running.
  #-----------------------------------------------------------
  if ( $sServerProfile =~ m|^/| )
  {
    &logTrace(20064,"<%s> : Forming PidFile: Complete path to ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
  }
  else
  {
    &logTrace(20065,"<%s> : Forming PidFile: Only name of ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    $sWASLogDir = "$sWASHome/$sWASLogDir";
  }
  my $sPidFile = "$sWASLogDir/$sServerName/$sServerName.pid"; 
  my $iReturn = 0 ; 
  my $sEnvFile = "";
  my $iSourceEnvFlag = 0;
  my $iExitCode  = 0;
  my @lsCommandOut = ();
  my $bTimedOut = 0;
  my @sServerNames = ();
  
  $iTimeOut = $iRelevantTimeout -(time - $iEntryPointStartTime) - 3;
  if ( $sServerType eq "Cluster" ){
    $iReturn = GetClusterMembers( $sServerName,
                                  $sServerProfile,
                                  $sUser,
                                  $sDMHostName,
                                  $iDMPort,
                                  $sWASBinDir,
                                  $iTimeOut,
                                  \@sServerNames );
    if ( $iReturn ){
      &logError( 20278, "<%s>: Error getting list of cluster members.", $sMyName );
      $iReturnCode = 3;
    }
    else{
      my $sResourceName = $ENV { VCS_LOG_RESOURCE_NAME };
      my $sCommand = $sCmdHares;
      my $sCommandArgs = "-value $sResourceName Group";
      my ($sGrpName, @lsHagrpOut);
      if ( ! -e $sCommand ) {
        &logTrace ( 20295,"<%s>:Failed:Cmd [%s] does not exist.", $sMyName,$sCommand );
      }
      elsif ( ! -x $sCommand ) {
        &logTrace ( 20296,"<%s> Failed:Cmd [%s] is not executable.", $sMyName,$sCommand );
      }

      #---------------------------------------------
      # Assume the group name is valid and proceed..
      #---------------------------------------------
      my @lsHaresOut = grep { s/\n// || 1 } readpipe ( "$sCommand $sCommandArgs" );

      #-----------------------------------------
      # Check if command executed successfully..
      #-----------------------------------------
      if ( $iExitCode = $? >> 8 ) {
        &logError ( 20297,"<%s> Cmd [%s] with args [%s] failed with exit code [%s]", $sMyName, $sCommand, $sCommandArgs, $iExitCode );
        $iReturnCode = 3;
      }
      else {
        $sGrpName = $lsHaresOut[0] ;
        &logTrace ( 20298,"<%s> Group for resource [%s] is [%s].", $sMyName,$sResourceName,$sGrpName );
      }
      my @sSystemList;
      my ( $iRunningCount, $iTempCount ) = 0;
      $sCommand = $sCmdHagrp;
      $sCommandArgs = "-state $sGrpName";
      &logTrace ( 20299,"<%s> Running command [%s] with arguments [%s].",     $sMyName, $sCommand, $sCommandArgs );
      @lsHagrpOut = grep { (s/\n// || 1) } readpipe ( "$sCommand $sCommandArgs" );

      #-----------------------------------------
      # Check if command executed successfully..
      #-----------------------------------------
      if ( $iExitCode = $? >> 8 ) {
        &logError ( 20300,"<%s> Cmd [%s] with args [%s] failed with exit code [%s]", $sMyName, $sCommand, $sCommandArgs, $iExitCode );
        $iReturnCode = 3;
      }
      else {
        shift @lsHagrpOut;
        foreach my $sOut (@lsHagrpOut) {
          my @lsOut = split( '\s+', $sOut);
          push @sSystemList, $lsOut[2];
        }
        &logTrace ( 20301,"<%s> SystemList for group [%s] is [%s].", $sMyName, $sGrpName, join(',',@sSystemList) );
      }
      
      my $sCurrentHost = "";
      if ( GetNodeId( \$sCurrentHost ) ){
        &logError ( 20274, "<%s>: Couldn't retrieve node name.", $sMyName );
        return 0;
      }
      else{
        &logTrace( 20282,"<%s>: Current host is [%s]", $sMyName, $sCurrentHost );
      }

      my ( $iNumberOfMatches, $sFilter1, $sFilter2, @lsOutput, $sLine, $iRunningMemberCount, $sRegexWASName, @sLineArray, $sSystem, $sServer ) = ();
      $iRunningMemberCount = 0;
      
      foreach $sSystem ( @sSystemList ){
        if ( $sSystem ne $sCurrentHost ){
          @lsOutput = readpipe( "$sCmdSSH $sSystem \"$sCommandPs $sCommandPsArgs | $sCommandGrep -i java | $sCommandGrep -v grep\"" );
          if ( $iExitCode = $? >> 8 ){
            &logError ( 20283,"<%s>: Cmd [%s] with args [%s] failed with exit code [%s].", $sMyName, $sCommand, $sCommandArgs, $iExitCode );
          }
        }
        else{
          @lsOutput = readpipe( "$sCommandPs $sCommandPsArgs | $sCommandGrep -i java | $sCommandGrep -v grep" );
          if ( $iExitCode = $? >> 8 ){
            &logError ( 20283,"<%s>: Cmd [%s] with args [%s] failed with exit code [%s].", $sMyName, $sCommand, $sCommandArgs, $iExitCode );
          }
        }

        foreach $sServer ( @sServerNames ){
          # -------------------------------------
          # Form the filter to match WAS process
          #--------------------------------------
          $sRegexWASName = GetServerNameRegex($sServer);
          $iNumberOfMatches = 0;
          $sRegexWASName =~ s/\s+/\\s+/g;
          $sFilter1 = $sWASHome."/.*/bin/.*java.*"."com.ibm.ws.runtime.WsServer".".*"."\\s+".$sRegexWASName."\\s*\$";
          $sFilter2 = $sWASHome."/.*java.*-Dwas.launcher.server=\\w+:$sWASNode:$sRegexWASName";
          foreach $sLine ( @lsOutput ){
            if ( $sLine =~ /$sFilter1/ || $sLine =~ /$sFilter2/ ){
              $iNumberOfMatches += 1;
              &logTrace( 20276, "<%s>: Process for cluster member [%s] matched on line [%s]", $sMyName, $sServer, $sLine );
              last;
            }
          }
          if ( $iNumberOfMatches ){ 
            $iRunningMemberCount += 1;
            &logTrace( 20291, "<%s>: Cluster Member [%s] is running. Total [%s] cluster members running.",                                                                       $sMyName, $sServer, $iRunningMemberCount );
          }
        }
      }

      if ( $iRunningMemberCount >= $iMinClusterMember){
        $iReturnCode = 1;
      }
      else{
        $iReturnCode = 0;
      }
      &logTrace( 20292, "<%s>: Total cluster members running in first level monitoring: [%s]", $sMyName, $iRunningMemberCount );
    }
    return $iReturnCode;
  }
  else{
    $iReturn = open (FH,"$sPidFile");

    if ( ! $iReturn )
    {
#-------------------------
# Error opening pid file.
#-------------------------
      &logTrace(20066,"<%s>: Cannot open pid file [%s]",$sMyName,$sPidFile);

#---------------------------------------------------------------
# Because of HP-UX "ps" handling of long command lines we ignore 
# this particular check for HP-UX.
#---------------------------------------------------------------
      if ( Arch::HPUX_OS != $sOSType )
      {
        $iReturn = IsWASProcessRunning($sWASHome,$sWASNode,$sServerName);

        if ( 1 == $iReturn )
        {
#--------------------------------------------------
# Got valid process running for this WAS instance.
#--------------------------------------------------
          $iReturnCode = 1;
        }
        elsif ( 2 == $iReturn )
        {
#-----------------------------------------
# Zombie running for this WAS instance.
#-----------------------------------------
          $iReturnCode = 2;
        }
        elsif ( 0 == $iReturn )
        {
#--------------------------------------------
# No processes running for this WAS instance.
#--------------------------------------------
          $iReturnCode = 0;
        }
      }
      else
      {
#-----------------------------------------------------------
# We are on HP-UX. Cannot determine the status of WebSphere
# server in the absence of a pid file.
#-----------------------------------------------------------
        $iReturnCode = 0;
      }
    }
    else
    {
#--------------------
# PidFile is opened.
#--------------------
      my $iPid = <FH>;
      &logTrace(20067,"<%s>:Pidfile [%s] contains [%s]",
          $sMyName,$sPidFile,$iPid);

#--------------------------------------------------------
# Check this pid corresponds to the right Server Process.
#--------------------------------------------------------
      $iReturn = CheckWASProcessID($iPid,
          $sWASHome,
          $sWASNode,
          $sServerName,
          $sUser,
          $iTimeOut);
      if ( 1 == $iReturn )
      {
#-----------------------------------------------
# Good pid. Matches with the server processes !
#-----------------------------------------------
        &logTrace(20068,"<%s>:Pidfile [%s] contains correct running process.",
            $sMyName , $sPidFile);
        $iReturnCode = 1;
      }
      elsif ( 2 == $iReturn )
      {
#----------------------------------
# Matching process, but a zombie !
#----------------------------------
        &logTrace(20069,"<%s>:Pidfile [%s] contains zombie process.",
            $sMyName , $sPidFile);
        $iReturnCode = 2;
      }
      elsif ( 3 == $iReturn )
      {
#------------------------------------------------------------
# The PidFile is stale. A server is running but PidFile does
# not contain the correct pid.
#------------------------------------------------------------
        &logTrace(20070,"<%s>:PidFile [%s] contains non existant [%s] process.",
            $sMyName,$sPidFile,$iPid);
        $iReturnCode = 1;
      }
      elsif ( 4 == $iReturn )
      {
#-----------------------------------------------
# Process information could not be determined.
#-----------------------------------------------
        &logTrace(20071,"<%s>:Problem determining process status.",
            $sMyName,$sPidFile,$iPid);
        $iReturnCode = 3;
      }

      elsif ( 0 == $iReturn )
      {
#---------------------------------------
# No process running for this instance !
#---------------------------------------
        $iReturnCode = 0;
      }
    }
  }
  return $iReturnCode;
}

=pod

--------------------------------------------------------------------------------

 Function : Int IsWASProcessRunning ( IN  String WASHome ,
                                      IN  String WASNode ,
                                      IN  String ServerName ,
                                      OUT ListReference PIDs );

 Description : Determines whether WAS process is running.
 Applies the filter formed by WASHome,WASNodeServerName.

 Parameters :
 String  WASHome                WAS installation home
 String  WASNode                WAS Node
 String  ServerName             WAS Server name
 ListRef PIDs                   Reference to PIDs list

 Returns : Integer value as defined below:

 0  No processes running for this WAS instance.
 1  Processes found to be running for this WAS instance.
 ListReference of PIDs gets populated if process is running with its id.

--------------------------------------------------------------------------------

=cut

sub IsWASProcessRunning
{

  my ($sWASHome,
      $sWASNode,
      $sServerName,
      $rliPIDs    ) = @_;

  #-----------------
  # Announce Entry.
  #-----------------
  my ($sMyName ) = (caller 0)[3];
  &logTrace( 20072, "Subroutine: <%s> called with args
        WASHome      [%s]
        WASNode      [%s]
        ServerName   [%s]
        liPIDs       [%s]",
        $sMyName,$sWASHome, $sWASNode, $sServerName,$rliPIDs);

  my $iReturnCode   = 1; 

  #----------------------------------------------
  # Get all running processes and store in hash.
  #----------------------------------------------
  my @hProcListHash = ();
  my $iProcessCount = Proc::GetLongProcessListHash(\@hProcListHash);

  if ( 0 == $iProcessCount )
  {
    #----------------------
    # No processes found.
    #----------------------
    &logError(20073,"<%s> : Error getting all running processes.",$sMyName);
    return 0;
  }

  #--------------------------------------
  # Form the filter to match WAS process
  #--------------------------------------
  my $sRegexWASName = GetServerNameRegex($sServerName);
  $sRegexWASName =~ s/\s+/\\s+/g;
  my $sFilter = "^".$sWASHome."/.*/bin/.*java.*"."com.ibm.ws.runtime.WsServer".".*"."\\s+".$sWASNode."\\s+".$sRegexWASName."\\s*\$";
  my $sOSType = Arch::GetOSType();
  if ( Arch::HPUX_OS == $sOSType )
  {
    $sFilter = "^".$sWASHome.".*/bin/.*java.*";
  }
  &logTrace(20074,"<%s>:Filter [%s] set.",$sMyName,$sFilter);
  my @hProcListHashMatched = ();

  my $iNumberOfMatches = Proc::FilterProcs($sFilter,
                                           \@hProcListHash,
                                           \@hProcListHashMatched);
  if ( 0 >= $iNumberOfMatches )
  {
    $sFilter = "^".$sWASHome."/.*java.*-Dwas.launcher.server=\\w+:$sWASNode:$sRegexWASName";

    $iNumberOfMatches = Proc::FilterProcs($sFilter,
                                          \@hProcListHash,
                                          \@hProcListHashMatched);

    &logTrace(20331,"<%s>: [%s] Processes found with filter [%s]",
      $sMyName , $iNumberOfMatches , $sFilter);
  }

  if ( 0 >= $iNumberOfMatches )
  {
    #--------------------
    # No matches found.
    #--------------------
    &logTrace(20075,"<%s>: No processes found for [%s] websphere server.",
      $sMyName, $sServerName );
    return 0;
  }
  else
  {
    #------------------------------------------
    # Resource's processes found to be running.
    #------------------------------------------
    &logTrace(20076,"<%s>: [%s] Processes found with filter [%s]",
      $sMyName , $iNumberOfMatches , $sFilter);

    foreach my $hEachProcHash ( @hProcListHashMatched)
    {
      my $iTmpPPID = $hEachProcHash->{"PPID"};
      my $iTmpPID  = $hEachProcHash->{"PID"};
      my $iTmpCMD  = $hEachProcHash->{"COMMAND"};
      push(@$rliPIDs,$iTmpPID);
    }
    return 1;
  }
}

=pod

--------------------------------------------------------------------------------

 Function : Int CheckWASProcessID   ( IN Int    Pid,
                                      IN String WASHome ,
                                      IN String WASNode ,
                                      IN String ServerName ,
                                      IN String User ,
                                      IN Int    TimeOut );

 Description : Determine whether a pid is actually this WAS server's
 process. Applies the filter formed by WASHome,WASNode,ServerName.

 Parameters :
 Int     Pid                    Pid integer
 String  WASHome                WAS installation home
 String  WASNode                WAS Node
 String  ServerName             WAS Server name
 String  User                   Unix User Name
 Int     TimeOut                TimeOut

 Returns : Integer value as defined below:

 0  Pid does not represent correct WAS process.
 1  Pid does represent correct WAS process.
 2  Pid is zombie.
 3  Pid does not represent correct WAS process but a valid process for this
    WAS instance does exist on the system.
 4  Process state could not be determined

--------------------------------------------------------------------------------

=cut

sub CheckWASProcessID 
{

  my ($iPid,
      $sWASHome,
      $sWASNode,
      $sServerName,
      $sUser,
      $iTimeOut) = @_;

  #-----------------
  # Announce Entry.
  #-----------------
  my ($sMyName ) = (caller 0)[3];
  &logTrace( 20077, "Subroutine: <%s> called with
     Pid            [%s]
     WASHome        [%s]
     WASNode        [%s]
     ServerName     [%s]
     User           [%s]
     Timeout        [%s]",
     $sMyName, $iPid, $sWASHome, $sWASNode, $sServerName ,
     $sUser, $iTimeOut);

  my $sRegexWASName = GetServerNameRegex($sServerName);
  my $sFilter = $sWASHome.".*java.*"."\\s+".$sWASNode."\\s+".$sRegexWASName."\\s*\$";
  my $sAltFilter = $sWASHome."/.*java.*-Dwas.launcher.server=\\w+:$sWASNode:$sRegexWASName";

  #-----------------------------------------------
  # Set up correct ps command with arguments.
  # This need to be handled differently for each
  # platform.
  #-----------------------------------------------
  my $sOSType = Arch::GetOSType();

  my $sCommandPs = "/usr/bin/ps";
  my $sCommandArgsPs = "axwwl";

  if(Arch::SUN_OS == $sOSType)
  {
    my $sOSVer = ( uname ) [2]; # Get OS version..
    if ($sOSVer =~ /\d+\.(\d+)/) {
      $sOSVer = $1;
    }
    if ( $sOSVer < 11 ) {
      $sCommandPs = "/usr/ucb/ps" if ( -x "/usr/ucb/ps" );
    }
  }
  elsif ( Arch::HPUX_OS == $sOSType )
  {
    $sFilter = $sWASHome.".*java.*";
    $sCommandPs     = "/bin/ps";
    $ENV{"UNIX95"}  = 1;
    $sCommandArgsPs = "-xxo 'uid,pid,state,ppid,args' -p";
  } 
  elsif   (  Arch::LINUX_OS == $sOSType
          || Arch::AIX_OS   == $sOSType )
  {
    $sCommandPs = "/bin/ps";
    if ( Arch::LINUX_OS == $sOSType )
    {
      $sCommandArgsPs = "-wwo 'uid,pid,state,ppid,args' -p";
    }
    else
    {
      $sCommandArgsPs = "-o 'uid,pid,state,ppid,args' -p";
    }
  }
  $sCommandArgsPs = $sCommandArgsPs." ".$iPid;
  #---------------------------------------------------------
  # SUN   : [/usr/bin/ps axwwl $iPID]
  # SUN   : [/usr/ucb/ps axwwl $iPID] if ( $sOSVer < 11 )
  # LINUX : [/bin/ps -o 'uid,pid,state,ppid,args' -p $iPID]
  # AIX   : [/bin/ps -o 'uid,pid,state,ppid,args' -p $iPID]
  # HP    : [/bin/ps -xxo 'uid,pid,state,ppid,args' -p $iPID]
  #---------------------------------------------------------

  my $sEnvFile = "";
  my $iSourceEnvFlag = 0;
  my $iExitCode;
  my @lsCommandOut;
  my $bTimedOut;

  my $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut($sEnvFile,
                                                              $iSourceEnvFlag,
                                                              $sCommandPs,
                                                              $sCommandArgsPs,
                                                              \$iExitCode,
                                                              $sUser,
                                                              \@lsCommandOut,
                                                              \$bTimedOut,
                                                              $iTimeOut);
  if ( $iReturnCode )
  {
    #---------------------------
    # Command got executed.
    #---------------------------
    if ( $iExitCode )
    {
      #---------------------
      # Command failed !
      #---------------------
      &logTrace (20078,"<%s>:command [%s] with args [%s] failed with exit code [%s]",
                 $sMyName,$sCommandPs,$sCommandArgsPs,$iExitCode);


      #-------------------------------------------------------------------
      # Command may fail if the process is not running hence we need to 
      # again confirm whether other process is running for this server.
      #-------------------------------------------------------------------
      my $iReturn = IsWASProcessRunning($sWASHome,$sWASNode,$sServerName);
      if ( 1 == $iReturn )
      {
        #--------------------------------------------------
        # Got valid process running for this WAS instance.
        # This means the pid file is stale.
        #--------------------------------------------------
        if ( Arch::HPUX_OS == $sOSType )
        {
          &logWarn(20079,"<%s>:Pidfile is stale.",$sMyName);
          return 0;
        } 
        &logWarn(20080,"<%s>:Pidfile is stale.",$sMyName);
        return 3;
      }
    }
    else
    {
      #------------------------
      # Successful execution.
      #------------------------
      &logTrace(20081,"<%s>:command [%s] with args [%s] successfully executed with exit code [%s]",
                           $sMyName,$sCommandPs,$sCommandArgsPs,$iExitCode);


      #--------------------------------------------------------------
      # Now we need to see whether this pid is actually our server !
      #--------------------------------------------------------------
      my $sLine;
      foreach $sLine (@lsCommandOut)
      {
        if ( $sLine =~ /$sFilter/ || $sLine =~ /$sAltFilter/ )
        {
          &logTrace(20082,"<%s>:WAS process found [%s]",$sMyName,$sLine);
          my (@lsEntry) = split " " ,$sLine;
          if ( $lsEntry[2] eq "Z" )
          {
            #----------------------
            # Process is zombie !
            #----------------------
            &logWarn(20083,"<%s>:Process State [%s] is zombie.",
                     $sMyName,$lsEntry[20053]);
            return 2;
          }

          #------------------------------------------
          # Got proper process running for this pid.
          #------------------------------------------
          return 1;
        }
      }

      #-----------------------------------------------------------------------
      # We reached here because matching process not found with the given pid.
      # Now we have to check whether process is running with other pid.
      #-----------------------------------------------------------------------

      &logTrace(20084,"<%s>:matching process not found for pid.",$sMyName);
      my $iReturn = IsWASProcessRunning($sWASHome,
                                        $sWASNode,
                                        $sServerName);
      if ( 1 == $iReturn )
      {
        #--------------------------------------------------
        # Got valid process running for this WAS instance.
        # This means the pid file is stale.
        #--------------------------------------------------
        &logWarn(20085,"<%s>:Pidfile is stale.",$sMyName);
        if ( Arch::HPUX_OS == $sOSType )
        {
          &logWarn(20086,"<%s>:Pidfile is stale.",$sMyName);
          return 0;
        } 
        return 3;
      }
      return 0;
    }
  }
  else
  {
    if ( $bTimedOut )
    {
      #--------------------
      # Command timed out.
      #--------------------
      &logError (20087,"<%s>:command [%s] with args [%s] did not complete within [%s] seconds.",
      $sMyName,$sCommandPs,$sCommandArgsPs,$iTimeOut);
    }
    else
    {
      #--------------------
      # Unknown error.
      #--------------------
      &logError(20088,"<%s>:command [%s] with args [%s] execution is unknown.",$sMyName,$sCommandPs,$sCommandArgsPs);
    }
    return 4;
  }
}


=pod

--------------------------------------------------------------------------------

 Function : Int SecondLevelMonitor  ( IN String WASHome ,
                                      IN String WASNode ,
                                      IN String ServerName ,
                                      IN String ServerProfile,
                                      IN String ServerType,
                                      IN String DMHostName,
                                      IN integwe DMPort,
                                      IN inreger MinClusterMember,
                                      IN String WASBinDir ,
                                      IN String User ,
                                      IN Int    TimeOut );

 Description : Execute the serverStatus.sh command provided by IBM
 with ServerName as the argument. Check whether this IBM utility reports
 the WAS instance is running and in "STARTED" state.

 Parameters :
 String  WASHome                WAS installation home
 String  WASNode                WAS Node
 String  ServerName             WAS Server name
 String ServerProfile           Deployment manager Server profile
 String ServerType              WAS Server Type
 String DMHostName              WAS Deployment Manager hostname
 Integer DMPort                 WAS Deployment Manager SOAP Connector Port
 Integer MinClusterMember       Minimum number of cluster members running for cluster to be online
 String  WASBinDir              WAS bin directory
 String  User                   Unix User Name
 Int     TimeOut                TimeOut

 Returns : Integer value as defined below:

  99  Instance status is unknown.
 100  Instance is offline.
 110  Instance is online.

--------------------------------------------------------------------------------

=cut

sub SecondLevelMonitor 
{
  my ( $sWASHome,
       $sWASNode,
       $sServerName,
       $sServerProfile,
       $sServerType,
       $sDMHostName,
       $iDMPort,
       $iMinClusterMember,
       $sWASBinDir,
       $sUser,
       $iTimeOut ) =  @_;

  #-----------------
  # Announce Entry.
  #-----------------
  my $sMyName  = (caller 0)[3];
  &logTrace( 20089, "Subroutine: <%s> called with
             WASHome             [%s]
             WASNode             [%s]
             ServerName          [%s]
             ServerProfile       [%s]
             ServerType          [%s]
             DMHostName          [%s]
             DMPort              [%s]
             MinClusterMember    [%s]
             WASBinDir           [%s]
             User                [%s]
             TimeOut             [%s]",
             $sMyName , $sWASHome , $sWASNode , $sServerName, $sServerProfile, $sServerType, $sDMHostName, 
             $iDMPort, $iMinClusterMember, $sWASBinDir , $sUser , $iTimeOut);

  my $sEnvFile = "";
  my $iSourceEnvFlag = 0;
  my $iExitCode = 0;
  my @lsCommandOut = ();
  my $bTimedOut = 0;
  my $sEPTimeout;
  my $iReturn = 0;
  my @sServerNames = ();
  my $iRelevantTimeout = $iTimeOut;
  my $iEntryPointStartTime = $^T;
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;           #set return code optimistically to online

  #-------------------------------------------------------------------
  # Handle embedded spaces and (IBM supported) special characters 
  # in WAS Server Name. Need to take care of spaces and tabs when
  # executing a command with those characters.
  #-------------------------------------------------------------------
  my $sRegexWASName = GetServerNameRegex($sServerName);
  $sRegexWASName =~ s/ /\\ /g;
  $sRegexWASName =~ s/\t/\\\t/g;
  $sServerName = $sRegexWASName;

  if ( $sServerType eq "Cluster" ){
    #------------------------------------------------------------------------------
    # Command to query cluster state
    # wsadmin.sh -lang jython -conntype SOAP -host $sDMHostName -port $iDMPort -c "clusterName='$sServerName'; clusterObj = AdminControl.completeObjectName('type=Cluster,name='+clusterName+',*'); AdminControl.getAttribute(clusterObj, 'state' );" 
    #------------------------------------------------------------------------------

    my $sMonitorScript = "$sWASBinDir/".WebSphere::WAS_WSADMIN;
    my $sMonitorOptions = "-lang jython -conntype SOAP -host ".$sDMHostName." -port ".$iDMPort." -c \"clusterName=\'".$sServerName."\'; clusterObj = AdminControl.completeObjectName('type=Cluster,name='+clusterName+',*'); AdminControl.getAttribute(clusterObj, 'state' );\"";

    $iReturn = Sys::RunWithEnvCmdWithOutputWithTimeOut( $sEnvFile,
                                                   $iSourceEnvFlag,
                                                   $sMonitorScript,
                                                   $sMonitorOptions,
                                                   \$iExitCode,
                                                   $sUser,
                                                   \@lsCommandOut,
                                                   \$bTimedOut,
                                                   $iTimeOut,
                                                   "",
                                                   "$sWASBinDir" );


      #---------------------------------------------
      # Sample command output:
      # WASX7209I: Connected to process "dmgr" on node Dmgr02-node using SOAP connector;  The type of process is: DeploymentManager
      #  'websphere.cluster.stopped'
      #---------------------------------------------
    if ( $iReturn ){
      #----------------------
      # Command got executed.
      #----------------------
      if ( $iExitCode == 0 ){
        my $sClusterState = $lsCommandOut[1];
        &logTrace( 20379, "<%s>: [%s] command executed successfully. Found cluster in state: [%s]", $sMyName, $sMonitorScript, $sClusterState );
        #------------------------------------------------------------------------------------------------
        # Even if cluster state is partial.start or partial.stop,  we can mark the resource as online. 
        # Final resource state is decided based on minimum cluster members running count.
        #------------------------------------------------------------------------------------------------
        if( $sClusterState =~ /websphere.cluster.running/
         || $sClusterState =~ /websphere.cluster.partial/ ){
          $iReturnCode = VCSDef_MonitorExit::ONLINE;
        }
        else{
          $iReturnCode = VCSDef_MonitorExit::OFFLINE;
        }
      }
      elsif ( $iExitCode == 1 ){
        &logError( 20382, "<%s>: Specified profile cannot be found or profile not linked correctly.", $sMyName );
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
      }
      elsif ( $iExitCode == 103 ){
        &logError( 20383, "<%s>: Wrong Server Profile specified or Deployment Manager is not runnig or some error occured while executing [%s].", $sMyName, $sMonitorScript );
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
      }
      else{
        &logError( 20384, "<%s>: Unknown error occured. Command [%s] with args [%s] returned exit code [%s]", $sMyName,$sMonitorScript,$sMonitorOptions,$iExitCode );
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
      }
    }
    else{
      if ( $bTimedOut ){
        #-------------------
        # Command timed out.
        #-------------------
        &logError ( 20385,"<%s>:[%s] did not complete within [%s] seconds.", $sMyName, $sMonitorScript, $iTimeOut );
      }
      else
      {
        #-------------------
        # Command failed.
        #-------------------
        &logError ( 20386,"<%s>: Command [%s] failed.", $sMyName,$sMonitorScript );
      }
      $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
    }

    $iReturn = GetClusterMembers( $sServerName,
                                  $sServerProfile,
                                  $sUser,
                                  $sDMHostName,
                                  $iDMPort,
                                  $sWASBinDir,
                                  $iTimeOut,
                                  \@sServerNames );
    if ( $iReturn ){
      &logError( 20278, "<%s>: Error getting list of cluster members.", $sMyName );
      $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
    }
    else{
    &logTrace( 20303,"<%s>: Cluster Members list: [%s]", $sMyName, join( ',', @sServerNames ) );

    my $sServer = "";
    my $iRunningCount = 0;
    foreach $sServer (@sServerNames){
      #------------------------------------------------------------------
      # Command to query each member state
      # wsadmin.sh -lang jython -conntype SOAP -host $sDMHostName -port $iDMPort -c "serverName='$sServer\'; serverObj = AdminControl.completeObjectName('type=Server,name='+serverName+',*'); AdminControl.getAttribute(serverObj, 'state' );"
      #-----------------------------------------------------------------
      $sMonitorOptions = "-lang jython -conntype SOAP -host ".$sDMHostName." -port ".$iDMPort." -c \"serverName=\'".$sServer."\'; serverObj = AdminControl.completeObjectName('type=Server,name='+serverName+',*'); AdminControl.getAttribute(serverObj, 'state' );\"";
    &logTrace( 20304, "<%s>:Going to run command [%s] with arguments [%s].", $sMyName,$sMonitorScript,$sMonitorOptions);
#  @lsOutput = grep { (s/\n// || 1) } readpipe ( "$sMonitorScript $sMonitorOptions" );
    $iTimeOut = $iRelevantTimeout -(time - $iEntryPointStartTime) - 3;
    my $iTemp_Return = Sys::RunWithEnvCmdWithOutputWithTimeOut( $sEnvFile,
                                                                $iSourceEnvFlag,
                                                                $sMonitorScript,
                                                                $sMonitorOptions,
                                                                \$iExitCode,
                                                                $sUser,
                                                                \@lsCommandOut,
                                                                \$bTimedOut,
                                                                $iTimeOut,
                                                                "",
                                                                "$sWASBinDir" );
    if ( $iTemp_Return ){
      #----------------------
      # Command got executed.
      #----------------------
      if ( $iExitCode == 0 ){
        &logTrace( 20379, "<%s>: [%s] command executed successfully.", $sMyName, $sMonitorScript );
      }
      elsif ( $iExitCode == 1 ){
        &logError( 20382, "<%s>: Specified profile cannot be found or profile not linked correctly.", $sMyName );
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
      }
      elsif ( $iExitCode == 103 ){
        &logError( 20383, "<%s>: Wrong Server Profile specified or Deployment Manager is not runnig or some error occured while executing [%s].", $sMyName, $sMonitorScript );
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
      }
      else{
        &logError( 20384, "<%s>: Unknown error occured. Command [%s] with args [%s] returned exit code [%s]", $sMyName,$sMonitorScript,$sMonitorOptions,$iExitCode );
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
      }
    }
    else{
      if ( $bTimedOut ){
        &logError ( 20385,"<%s>:[%s] did not complete within [%s] seconds.", $sMyName, $sMonitorScript, $iTimeOut );
      }
      else
      {
        &logError ( 20386,"<%s>: Command [%s] failed.", $sMyName,$sMonitorScript );
      }
      $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
    }

      #---------------------------------------------
      # Sample command output if server is not running:
      # WASX7209I: Connected to process "dmgr" on node Dmgr02-node using SOAP connector;  The type of process is: DeploymentManager
      #WASX7015E: Exception running command: "serverObj = AdminControl.completeObjectName('type=Server,name='+ 'server3' + ',*'); state = AdminControl.getAttribute(serverObj, 'state');"; exception information:com.ibm.ws.scripting.ScriptingException: WASX7025E: Error found in String ""; cannot create ObjectName.
      #---------------------------------------------
      # Sample command output if server is running:
      # WASX7209I: Connected to process "dmgr" on node Dmgr02-node using SOAP connector;  The type of process is: DeploymentManager
      # 'STARTED'
      #---------------------------------------------
    if ( $iExitCode == 0 ){  
      if( $lsCommandOut[1] =~ /STARTED/ ){
        $iRunningCount += 1;
        &logTrace( 20305, "Server [%s] is running. Online members count: [%s]", $sServer, $iRunningCount );
      }
      elsif( $lsCommandOut[1] =~ /WASX7025E/ || $lsCommandOut[1] =~ /WASX7015E/){
        &logInfo ( 20306, "<%s>:Server [%s] is stopped. Online members count: [%s]", $sMyName, $sServer, $iRunningCount );
      }
      else{
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
      }
    }
    }

    if( $iRunningCount >= $iMinClusterMember ){
      $iReturnCode = VCSDef_MonitorExit::ONLINE;
    }
    else{
      $iReturnCode = VCSDef_MonitorExit::OFFLINE;
    }
    &logTrace( 20292, "<%s>: Total cluster members running after second level monitoring: [%s]", $sMyName, $iRunningCount );
    }
    return $iReturnCode;
  }
  else{
  #-------------------------------------------
  # Form the absolute path of serverStatus.sh 
  #-------------------------------------------
  my $sServerStatusCmd = "$sWASBinDir/".WebSphere::WAS_STATUS;
  &logTrace(20090,"ServerStatus Cmd is [%s]",$sServerStatusCmd);

  my $sServerStatusArgs = "$sServerName -logfile /dev/null";

  $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut($sEnvFile,
                                                  $iSourceEnvFlag,
                                                  $sServerStatusCmd,
                                                  $sServerStatusArgs,
                                                  \$iExitCode,
                                                  $sUser,
                                                  \@lsCommandOut,
                                                  \$bTimedOut,
                                                  $iTimeOut,
                                                  "",
                                                  "$sWASBinDir");

  if ( $iReturnCode )
  {
    #-----------------------
    # Command got executed.
    #-----------------------
    if ( $iExitCode )
    {
      #------------------
      # Command failed.
      #------------------
      &logError (20091,"<%s>:[%s] failed with exit code [%s]",
                     $sMyName,$sServerStatusCmd,$iExitCode);
      VCSagentFW::logOutputFromCommand("$sServerStatusCmd $sServerStatusArgs",
                                  $iExitCode,
                                  \@lsCommandOut,
                                  LogInt::LLError);
      return VCSDef_MonitorExit::UNKNOWN;
    }
    else
    {
      #--------------------------------
      # Command executed successfully.
      #--------------------------------
      &logTrace (20092, "<%s>:[%s] executed successfully.",
                 $sMyName, $sServerStatusCmd);
      
      foreach my $sOutputLine ( @lsCommandOut )
      {
        #------------------------------------------------
        # If the Server is reported as running then only
        # return 1 else return 0.
        #------------------------------------------------
        if( $sOutputLine =~ /ADMU0508I/ )
        {
          &logTrace(20093,"<%s>:serverStatus.sh reports server running.",$sMyName);
          return VCSDef_MonitorExit::ONLINE;
        }
      }   
      &logError(20094,"<%s>:serverStatus.sh reports server not running.",$sMyName);
      VCSagentFW::logOutputFromCommand("$sServerStatusCmd $sServerStatusArgs",
                                  $iExitCode,
                                  \@lsCommandOut,
                                  LogInt::LLError);
      return VCSDef_MonitorExit::OFFLINE;
    }
  }
  else
  {
    #--------------------
    # Command timed out.
    #--------------------
    if ( $bTimedOut )
    {
      # Command timed out.
      &logError (20095,"<%s>:[%s] did not complete within [%s] seconds.",
                 $sMyName,$sServerStatusCmd,$iTimeOut);
      VCSagentFW::logOutputFromCommand("$sServerStatusCmd $sServerStatusArgs",
                                  $iExitCode,
                                  \@lsCommandOut,
                                  LogInt::LLError);
    }
    else
    {
      #--------------------
      # Command failed.
      #--------------------
      &logError (20096,"<%s>:[%s] failed.",
                 $sMyName,$sServerStatusCmd);
      VCSagentFW::logOutputFromCommand("$sServerStatusCmd $sServerStatusArgs",
                                  $iExitCode,
                                  \@lsCommandOut,
                                  LogInt::LLError);
    }
    return VCSDef_MonitorExit::UNKNOWN;
  }
  }
  return $iReturnCode;
}

=pod

--------------------------------------------------------------------------------

 Function : Boolean Kill ( IN String $sWASHome ,
                           IN String $sServerName ,
                           IN String $sWASNode ,
                           IN String $sServerType ,
                           IN String DMHostName,
                           IN integer DMPort 
                           IN String $sServerProfile,
                           IN String $sUser,
         IN Integer $iTimeOut );

 Description : Kills all running WAS processes for this instance.
 Removes the PidFile if it exists.

 Parameters :
 String $sWASHome          Absolute Path to WAS install home
 String $sServerName       WAS Server Name
 String $sWASNode          Host on which WAS is configured
 String $sServerType       WAS Server Type
 DMHostName             Deployment Manager Host name
 DMPort                SOAP Connector port of Deployment Manager
 String $sServerProfile    WAS Server Profile name (only 6x)
 String $sUser             Unix User name
 Integer $iTimeOut         Entry point timeout

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true

--------------------------------------------------------------------------------

=cut

sub Kill
{
  my ( $sWASHome ,
       $sServerName ,
       $sWASNode ,
       $sServerType ,
       $sDMHostName,
       $iDMPort,
       $sServerProfile,
       $sUser,
       $iTimeOut ) = @_;

  #-------------------------------------------------------------------
  # $iReturnCode: Return code of sub Kill()
  # $iReturn: Return code of Sys::RunWithEnvCmdWithOutputWithTimeOut()
  #-------------------------------------------------------------------
  my $iReturnCode = 1;
  my $iReturn = 0 ; 

  #----------------
  # Announce Entry
  #----------------
  my $sMyName = ( caller 0 )[3];
  &logTrace( 20097, "Subroutine [%s] called with 
             WASHome         [%s]
             ServerName      [%s]
             WASNode         [%s]
             ServerType      [%s]
       DMHostName      [%s]
       DMport          [%s]
       ServerProfile   [%s]
       User       [%s]
       TimeOut       [%s]",
             $sMyName,$sWASHome,$sServerName,$sWASNode,$sServerType, $sDMHostName, $iDMPort, $sServerProfile, $sUser,$iTimeOut );

  my $sWASLogDir = "logs";
  my $sWASBinDir = "bin";
  my $iRelevantTimeout = $iTimeOut;
  my $iEntryPointStartTime = $^T;
  if ( $sServerProfile )
  {
    #-----------------------------------------
    # This is a version 6.x or above instance.
    #-----------------------------------------
    &logTrace(20102,"<%s> : Server version is 6.x and above",$sMyName);

    if ( $sServerProfile =~ m|^/| )
    {
      $sWASLogDir = "$sServerProfile/logs";
      $sWASBinDir = "$sServerProfile/bin";
      &logTrace(20103,"<%s> : Complete path to ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    }
    else
    {
      $sWASLogDir = "profiles/$sServerProfile/logs";
      $sWASBinDir  = "profiles/$sServerProfile/bin";
      &logTrace(20104,"<%s> : Only name of ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
    }
  }

  my $sEnvFile = "";
  my $iSourceEnvFlag = 0;
  my $iExitCode;
  my @lsCommandOut;
  my $bTimedOut;
  
  if ( $sServerProfile =~ m|^/| )
  {
    &logTrace(20105,"<%s> : Forming PidFile: Complete path to ServerProfile [%s] is specified. ",$sMyName, $sServerProfile);
  }
  else
  {
    &logTrace(20106,"<%s> : Forming PidFile: Only name of ServerProfile [%s] is specified or this is version 5x. ",$sMyName, $sServerProfile);
    $sWASLogDir = "$sWASHome/$sWASLogDir";
  }
  my $sPidFile = "$sWASLogDir/$sServerName/$sServerName.pid"; 

  $iTimeOut = $iRelevantTimeout -(time - $iEntryPointStartTime) - 3;
  if ( $sServerType eq "Cluster" ){
    my $sKillScript = "$sWASBinDir/".WebSphere::WAS_WSADMIN;
    if (! -x $sKillScript )
    {
      &logError( 20387,"<%s>:Kill script [%s] is not executable. Check file existence and permissons.",$sMyName,$sKillScript );
      return 0;
    }

      #--------------------------------------------------------------------------------------
      # Command to immediately stop the cluster
      # wsadmin.sh -lang jython -conntype SOAP -host $sDMHostName -port $iDMPort -c "clusterName='$sServerName'; clusterObj = AdminControl.completeObjectName('type=Cluster,name='+clusterName+',*'); AdminControl.invoke(clusterObj, 'stopImmediate' );"
      #-------------------------------------------------------------------------------------
    my $sKillOptions = "-lang jython -conntype SOAP -host ".$sDMHostName." -port ".$iDMPort." -c \"clusterName=\'".$sServerName."\'; clusterObj = AdminControl.completeObjectName('type=Cluster,name='+clusterName+',*'); AdminControl.invoke(clusterObj, 'stopImmediate' );\"";

    $iReturn = Sys::RunWithEnvCmdWithOutputWithTimeOut( $sEnvFile,
                                                        $iSourceEnvFlag, 
                                                        $sKillScript,
                                                        $sKillOptions,
                                                        \$iExitCode,
                                                        $sUser,
                                                        \@lsCommandOut,
                                                        \$bTimedOut,
                                                        $iTimeOut,
                                                        "",
                                                        "$sWASBinDir" );
    if ( $iReturn ){
      #-----------------------
      # Command got executed.
      #-----------------------
      if ( $iExitCode == 0 )
      {  
        &logTrace( 20388, "<%s>: Command [%s] executed successfully.", $sMyName, $sKillScript );
        $iReturnCode = 1
      }
      elsif ( $iExitCode == 1 ){
        &logError( 20389, "<%s>: Specified profile [%s] cannot be found or profile not linked correctly.", $sMyName, $sServerProfile );
        $iReturnCode = 0;
      }
      elsif ( $iExitCode == 103 ){
        &logError( 20390, "<%s>: Wrong Server Profile [%s] specified or Deployment Manager is not runnig or some error occured while executing [%s].", $sMyName, $sServerProfile,$sKillScript );
        $iReturnCode = 0;
      }
      else{
        &logError( 20391, "<%s>: Unknown error occured. Command [%s] with args [%s] returned exit code [%s]", $sMyName,$sKillScript,$sKillOptions,$iExitCode );
        $iReturnCode = 0;
      }
    }
    else{
      #--------------------
      # Command timed out.
      #--------------------
      if ( $bTimedOut )
      {
        # Command timed out.
        &logError ( 20392,"<%s>:[%s] did not complete within [%s] seconds.", $sMyName,$sKillScript,$iTimeOut);
      }
      else
      {
        #--------------------
        # Command failed.
        #--------------------
        &logError ( 20393, "<%s>:[%s] failed.", $sMyName,$sKillScript);
      }
        $iReturnCode = 0;
    }
  }
  elsif ( $sServerType eq "NodeAgent" || $sServerType eq "ApplicationServer" || $sServerType eq "DeploymentManager" ){
    my @liScriptPIDs = ();
    my $iReturn = GetScriptPIDs($sWASHome,$sServerName,$sWASNode,$sServerType,\@liScriptPIDs);
    if ( $iReturn )
    {
      &logTrace(20098,"<%s>:WAS scripts running for this server.",$sMyName);
      my $blongproclist = 1;
      if (Proc::KillPidsFromList(\@liScriptPIDs,$blongproclist))
      {
        &logTrace(20099,"<%s>:Killed WAS script processes.",$sMyName);
      }
      else
      {
        &logError(20100,"<%s>:Error killing WAS script processes.",$sMyName);
        $iReturnCode = 0;
      }
    }
    else
    {
      &logTrace(20101,"<%s>:WAS scripts not running for this server.",$sMyName);
    }

    my $sOSType = Arch::GetOSType();
    if ( Arch::HPUX_OS != $sOSType )
    {
      my @liPIDs = ();
      $iReturn = IsWASProcessRunning($sWASHome,
          $sWASNode,
          $sServerName,
          \@liPIDs);
      if ($iReturn)
      {
        &logTrace(20107,"<%s>:[%s] WAS Server PIDs [%s] found to be killed",
            $sMyName , $#liPIDs , @liPIDs);
        foreach my $itmp ( @liScriptPIDs )
        {
          push (@liPIDs, $itmp);
          &logTrace(20108,"Pushing script pids [%s]",$itmp);
        }
        my $blongproclist = 1;
        if (Proc::KillPidsFromList(\@liPIDs,$blongproclist))
        {
          &logTrace(20109,"<%s>:Killed WAS processes.",$sMyName);
        }
        else
        {
          &logError(20110,"<%s>:Error killing WAS processes.",$sMyName);
          $iReturnCode = 0;
        }
      }
      else
      {
        &logTrace(20111,"<%s>:No WAS Server process to be killed",$sMyName);
      }
    }
    else
    {
#-------------------------------------------------
# HP-UX , need to kill process from the pid file.
#-------------------------------------------------
      my $iReturn = 0 ; 
      $iReturn = open (FH,"$sPidFile");
      if ( ! $iReturn )
      {
        &logTrace(20112,"<%s>: Could not open pid file [%s], cannot kill WAS process.",$sMyName,$sPidFile);
      }
      else
      {
        my $iPid = <FH>;
        &logTrace(20113,"<%s>: Killing process [%s]",$sMyName,$iPid);
        kill 9, $iPid;
      }
    }

#-------------------------------------------------
# Remove SLM counter File of this WAS instance.
#-------------------------------------------------
    my $sResourceType   = $ENV{VCS_LOG_AGENT_NAME};
    my $sResourceName   = $ENV{VCS_LOG_RESOURCE_NAME};
    &logTrace(20114,"<%s>: Unlinking [%s]",$sMyName,"/tmp/.VRTS$sResourceType/$sResourceName.slm");
    unlink ("/tmp/.VRTS$sResourceType/$sResourceName.slm");

#----------------------------------------
# Remove Pid File of this WAS instance.
#----------------------------------------
    &logTrace(20115,"<%s>:PidFile [%s]",$sMyName,$sPidFile);
    if ( -e $sPidFile )
    {
      if (unlink ($sPidFile) )
      {
        &logTrace(20116,"<%s>:PidFile [%s] successfully removed.",$sMyName,$sPidFile);
      }
      else
      {
        &logError(20117,"<%s>:PidFile [%s] unable to remove.",$sMyName,$sPidFile);
        $iReturnCode = 0;
      }
    }
    else
    {
      &logTrace(20118,"<%s>:PidFile [%s] not present - no need to remove.",$sMyName,$sPidFile);
    }
  }
    return $iReturnCode;
}


=pod

--------------------------------------------------------------------------------

 Function : Boolean GetScriptPIDs ( IN  $sWASHome ,
                                    IN  $sServerName ,
                                    IN  $sWASNode ,
                                    IN  $sServerType,
                                    OUT $rliScriptPIDs );

 Description : Get all (start or stop) script processes running for this
 WAS instance.

 Parameters :
 $sWASHome      Absolute Path to WAS install home
 $sServerName   WAS Server Name
 $sWASNode      Host on which WAS is configured
 $sServerType   WAS Server Type
 $rliScriptPIDs Reference to list of script pids

 Returns : boolean value in the set of {0,1} where
           0 script processes not found running for this server.
           1 script processes found to be running for this server.
           rliScriptPids gets populated if scripts are running with their pids.
--------------------------------------------------------------------------------

=cut

sub GetScriptPIDs
{
  my ( $sWASHome ,
       $sServerName ,
       $sWASNode ,
       $sServerType,
       $rliScriptPIDs ) = @_;

  my $iReturnCode = 0;

  #----------------
  # Announce Entry
  #----------------
  my $sMyName = ( caller 0 )[3];
  &logTrace( 20119, "Subroutine [%s] called with 
             WASHome         [%s]
             ServerName      [%s]
             WASNode         [%s]
             ServerType      [%s]",
             $sMyName,$sWASHome,$sServerName,$sWASNode,$sServerType);

  my $sStartScript;
  my $sStopScript;
  #------------------------------------------------------------------
  # Not using ^ to start WASHome in the filter as the start scripts 
  # are run as /bin/sh /ibm.....
  #------------------------------------------------------------------
  my $sStartFilter = $sWASHome;
  my $sStopFilter  = $sWASHome;
  #------------------------------------------------------------------
  # Filter ends with any of start/stop script options for a WAS
  # instance or End of Line.
  #------------------------------------------------------------------
  if ( $sServerType eq "ApplicationServer")
  {
    my $sStartOptions = "-nowait|-quiet|-logfile|-replacelog|-trace|-script|-timeout|-statusport";
    my $sStopOptions  = "-nowait|-quiet|-logfile|-replacelog|-trace|-timeout|-statusport|-conntype|-port|-username|-password";
    $sStartScript = WebSphere::WAS_START_SERVER;
    $sStopScript  = WebSphere::WAS_STOP_SERVER;
    my $sRegexWASName = GetServerNameRegex($sServerName);
    $sRegexWASName =~ s/\s+/\\s+/g;
    $sStartFilter = $sStartFilter.".*".$sStartScript."\\s+".$sRegexWASName . "\\s*" . "($sStartOptions|\$)";
    $sStopFilter  = $sStopFilter.".*".$sStopScript."\\s+".$sServerName . "\\s*" . "($sStopOptions|\$)";
    &logTrace(20120,"<%s>:Start Filter set to [%s]",$sMyName,$sStartFilter);
    &logTrace(20121,"<%s>:Stop Filter set to [%s]",$sMyName,$sStopFilter);
  }
  elsif ( $sServerType eq "NodeAgent" )
  {
    my $sStartOptions = "-nowait|-quiet|-logfile|-replacelog|-trace|-script|-timeout|-statusport";
    my $sStopOptions = "-nowait|-stopservers|-quiet|-logfile|-replacelog|-trace|-timeout|-statusport|-conntype|-port|-username|-password";
    $sStartScript = WebSphere::WAS_START_NODE;
    $sStopScript  = WebSphere::WAS_STOP_NODE;
    $sStartFilter = $sStartFilter.".*".$sStartScript. "\\s*" . "($sStartOptions|\$)";
    $sStopFilter  = $sStopFilter.".*".$sStopScript . "\\s*" . "($sStopOptions|\$)";
    &logTrace(20122,"<%s>:Start Filter set to [%s]",$sMyName,$sStartFilter);
    &logTrace(20123,"<%s>:Stop Filter set to [%s]",$sMyName,$sStopFilter);
  }
  elsif ( $sServerType eq "DeploymentManager" )
  {
    my $sStartOptions = "-nowait|-quiet|-logfile|-replacelog|-trace|-script|-timeout|-statusport";
    my $sStopOptions = "-nowait|-quiet|-logfile|-replacelog|-trace|-timeout|-statusport|-conntype|-port|-username|-password";
    $sStartScript = WebSphere::WAS_START_MANAGER;
    $sStopScript  = WebSphere::WAS_STOP_MANAGER;
    $sStartFilter = $sStartFilter.".*".$sStartScript. "\\s*" . "($sStartOptions|\$)";
    $sStopFilter  = $sStopFilter.".*".$sStopScript . "\\s*" . "($sStopOptions|\$)";
    &logTrace(20124,"<%s>:Start Filter set to [%s]",$sMyName,$sStartFilter);
    &logTrace(20125,"<%s>:Stop Filter set to [%s]",$sMyName,$sStopFilter);
  }
  else
  {
    #-------------------------------------------------------------------------
    # Unsupported ServerType , cannot uniquely identify the script processes.
    #-------------------------------------------------------------------------
    &logTrace(20126,"<%s>:Cannot find script processes with ServerType [%s]",
              $sMyName,$sServerType); 
    return 0;
  }

  my @sFilterList = ($sStopFilter , $sStartFilter);
  my @rliScriptPIDs = ();
  my $bScriptFound = 0;

  foreach my $sFilter ( @sFilterList )
  {
    my @hProcListHashMatched = ();
    my @hProcListHash        = ();
    my $iMatchedProcCount    =  0;
    my $iProcessCount = Proc::GetLongProcessListHash(\@hProcListHash);

    if ( 0 == $iProcessCount )
    {
      #----------------------
      # No processes found.
      #----------------------
      &logError(20127,"<%s> : Error getting all running processes.",$sMyName);
      return 0;
    }
  
    my $iNumberOfMatches = Proc::FilterProcs( $sFilter, \@hProcListHash, \@hProcListHashMatched);
    &logTrace(20128,"<%s>:Found [%s] matches for Script Filter [%s]",$sMyName,$iNumberOfMatches,$sFilter);

    if ( 0 == $iNumberOfMatches )
    {
      &logTrace(20129,"<%s>:Script processes not found.",$sMyName);
    }
    else
    {
      #---------------------------------------
      # We have found the Start/Stop Script.
      #---------------------------------------
      $bScriptFound = 1;
      my @liStartPidList;
      $iMatchedProcCount = Proc::GetValuesForKeyFromProcDataHash( "PID", \@hProcListHashMatched, \@liStartPidList);
      if ( $iMatchedProcCount > 1)
      {
        &logError(20130,"<%s>:Multiple scripts [%s] running for this WAS instance.",$sMyName,$iMatchedProcCount);
      }
    
      # Take all PIDs of the script and find all the children
      # of the script to kill them.
      foreach my $sEachParent (@liStartPidList)
      {
        #---------------------------------------------------------
        # Push Start/Stop Script pid to list of pids to be killed.
        #---------------------------------------------------------
        push ( @$rliScriptPIDs,$sEachParent);
        foreach my $sEachChild ( @hProcListHash)
        {
          if ( $sEachChild->{"PPID"} == $sEachParent )
          {
            #------------------------------------
            # Found a child of the start script.
            #------------------------------------
            &logTrace(20131,"<%s>:Found child process of script.",$sMyName);
            &logTrace(20132,"Child Pid [%s]",$sEachChild->{"PID"});
            push (@$rliScriptPIDs , $sEachChild->{"PID"});
          }
        }
      }
      #-------------------------------------------------------------
      # Thus far we have gathered all pids of Start/Stop scripts 
      # running for this instance and also its children if any.
      # These pids are available to the caller via $rliScriptPIDs
      #-------------------------------------------------------------
    }
  }
  return $bScriptFound;
}

=pod

--------------------------------------------------------------------------------

 Function : Boolean SetSLMCounter ( IN String ResourceName ,
                                    IN Int    SLMCounterValue );

 Description : Set SLMCounter - temp VCS attribute for this resource
 to given value.

 Parameters :
 String  ResourceName           VCS Resource Name
 Int     SLMCounterValue        VCS temp attribute SLMCounter

 Returns : Boolean value as defined below:
           0  Second Level Monitoring failed.
           1  Second Level Monitoring passed.

--------------------------------------------------------------------------------

=cut

sub SetSLMCounter
{

  my ( $sResourceType, 
       $sResourceName , 
       $iSLMCounterValue ) = @_;

  #---------------
  # Announce Entry
  #---------------
  my $sMyName = ( caller 0 )[3];
  &logTrace( 20133, "Subroutine <%s> called with
             ResourceName    [%s]
             SLMCounter      [%s]",
             $sMyName, $sResourceName, $iSLMCounterValue );

  # open the file
  
  my $sDirName = "/tmp/.VRTS$sResourceType";
  mkdir ($sDirName,0700);
  if ( ! -d $sDirName )
  {
    &logError( 20134, "<%s>: Could not create directory [%s]",$sMyName,$sDirName);
  }
  my $sSLMCounterFile = "$sDirName/$sResourceName.slm";
  if ( !open ( SLMFILE , "> $sSLMCounterFile") )
  {
    &logError( 20135, "<%s>: Could not open file for writing [%s]",$sMyName,$sSLMCounterFile);
    return 0;
  }

  print SLMFILE $iSLMCounterValue;
  &logTrace(20136, "<%s>: Wrote [%s] into [%s] file.",$sMyName,$iSLMCounterValue,$sSLMCounterFile);
  return 1;
}

=pod

--------------------------------------------------------------------------------

 Function : Boolean GetSLMCounter ( IN String ResourceType ,
                                    IN String ResourceName )

 Description : Get SLMCounter - Sets the value of the SLM Counter file
 to given value.

 Parameters :
 String  ResourceType           VCS Resource Type
 String  ResourceName           VCS Resource Name

 Returns : Integer value

--------------------------------------------------------------------------------

=cut


sub GetSLMCounter
{

  my ( $sResourceType, 
       $sResourceName  ) = @_;

  #---------------
  # Announce Entry
  #---------------
  my $sMyName = ( caller 0 )[3];
  &logTrace( 20137, "Subroutine <%s> called with
             ResourceType    [%s]
             ResourceName    [%s]",
             $sMyName, $sResourceType, $sResourceName);

  my $sDirName = "/tmp/.VRTS$sResourceType";
  my $sSLMCounterFile = "$sDirName/$sResourceName.slm";
  &logTrace( 20138, "<%s>: SLM Counter file is [%s]",$sMyName,$sSLMCounterFile);
  if ( !open ( SLMFILE , "< $sSLMCounterFile") )
  {
    &logWarn( 20139, "<%s>: Could not open file [%s] for reading.",$sMyName,$sSLMCounterFile);
    return 0;
  }

  my $iSLMCounter = <SLMFILE>;
  $iSLMCounter =~ s/\s+//g;
  &logTrace(20140, "<%s>: Got [%s] from [%s] file.",$sMyName,$iSLMCounter,$sSLMCounterFile);
  return $iSLMCounter;
}

=pod

--------------------------------------------------------------------------------

 Function : String GetServerNameRegex ( IN String ServerName );

 Description : Prefixes the special characters within a  WAS server name
 with an extra "\" character.

 Parameters :
 String  ServerName

 Returns : String where the special characters are prefixed by an extra "\" .

--------------------------------------------------------------------------------

=cut

sub GetServerNameRegex
{
  my ($sServerName) = @_;
  $sServerName =~ s/\./\\\./g;
  $sServerName =~ s/\@/\\\@/g;
  $sServerName =~ s/\$/\\\$/g;
  $sServerName =~ s/\(/\\(/g;
  $sServerName =~ s/\)/\\)/g;
  $sServerName =~ s/\^/\\^/g;
  $sServerName =~ s/\[/\\[/g;
  $sServerName =~ s/\]/\\]/g;
  $sServerName =~ s/\{/\\{/g;
  $sServerName =~ s/\}/\\}/g;
  return $sServerName;
}

=pod
 -------------------------------------------------------------------------
 Function : int GetEPTimeout ( IN String EntryPointName
                               IN String ResourceType  );

 Description : Returns the Entry Point Timeout for the EP which we are in

 Parameters :

 $sEntryPointName           The EP which we are currently running
 $sResourceType             VCS resource type

 Returns  : Integer value as defined below:

 -------------------------------------------------------------------------
=cut

sub GetEPTimeout
{
  my ( $sEntryPointName, $sResourceType) = @_;

  #------------------
  # Announce Entry..
  #------------------

  my $sMyName = ( caller 0 )[3];
  &logTrace( 20167, "Subroutine <%s> called with:
  EntryPointName [%s]
  ResourceType   [%s]", 
  $sMyName , $sEntryPointName , $sResourceType );

  my $sObjectName          = "hatype";
  my %hTypeValues;

  my $iEngineTimeout;
  #-----------------------------------
  # Query if it has already been set..
  #-----------------------------------
  if ( defined $::giEPTimeout &&    # Defined..
       $::giEPTimeout =~ /^\d+/ ) { # to an integer..
    $iEngineTimeout = $::giEPTimeout;

    &logTrace ( 20168,
    "<%s> Using AEPTimeout [%s] as EP timeout value",
    $sMyName, $iEngineTimeout );
    return ( $iEngineTimeout ); 
  }

  if ( ! VCSagentFW::GetVCSObjectHash($sObjectName,$sResourceType,\%hTypeValues) )
  {
    &logError(20169,"<%s>:GetVCSObjectHash call failed.",$sMyName);
    return 0;
  }

  #-------------------------------
  # Make "monitor" to "Monitor"
  # offline to Offline and so on
  #-------------------------------

  my $sTempEntry =  $sEntryPointName ;
  $sTempEntry    =~ s/(\w)(.*)$/\U$1\L$2/; 

  &logTrace(20170,"<%s>: Returning [%s] value [%s]",$sMyName, $sTempEntry."Timeout",$hTypeValues{$sTempEntry."Timeout"});
  return ( $hTypeValues{$sTempEntry."Timeout"});
}

=pod

 -------------------------------------------------------------------------
 Function : int DelayEntryPoint ( IN String WASHome,
                                  IN String WASNode,
                                  IN String ServerName,
                                  IN String User,
                                  IN String ServerProfile,
                                  IN Strinf ServerType,
                                  IN String DMHostName,
                                  IN integer DMPort,
                                  IN integer MinClusterMember,
                                  IN Integer DelayToTime)

 Description : Returns the Entry Point Timeout for the EP which we are in.

 If we are in online , the subroutine delays the entry point till all 
 required WebSphere processes start running.

 If we are in offline , the subroutine delays the entry point till all 
 required WebSphere processes stop running.

 Parameters :

 $sWASHome          Absolute Path to WAS install home
 $sWASNode          Host on which WAS is configured
 $sServerName       WAS Server Name
 $sUser             Unix User for WAS instance
 $sServerProfile    WAS Server Profile name (only 6x)
 $sServerType        WAS Server Type
 $sDMHostName       Deployment Manager Host name
 $iDMPort           SOAP Connector port of Deployment Manager
 integer MinClusterMember    Minimum number of cluster members running for cluster to be online
 $iDelayToTime      Max time allowed to delay the EP

 Returns  : Integer value as defined below:
 0 Failure
 1 Success

 -------------------------------------------------------------------------

=cut

sub DelayEntryPoint
{
  my ( $sWASHome,
       $sWASNode,
       $sServerName,
       $sUser,
       $sServerProfile,
       $sServerType,
       $sDMHostName,
       $iDMPort,
       $iMinClusterMember,
       $iDelayToTime ) = @_ ;

  #------------------
  # Announce Entry..
  #------------------

  my $sMyName = ( caller 0)[3];
  &logTrace ( 20141, "Subroutine <%s> called with:
  WASHome       [%s]
  WASNode       [%s]
  ServerName    [%s]
  User          [%s]
  ServerProfile [%s]
  ServerType    [%s]
  DMHostName    [%s]
  DMPort        [%s]
  MinClusterMember [%s]
  DelayToTime   [%s]",
  $sMyName, $sWASHome, $sWASNode, $sServerName, $sUser, $sServerProfile, $sServerType, 
  $sDMHostName, $iDMPort, $iMinClusterMember, $iDelayToTime );

  my ($sEntryPointName, $sResourceType ) = reverse $0 =~ m@([^/]+)@g;

  my $iReturnCode;
  my $iNSR = $iDelayToTime - time;
  while ( $iNSR > CHECK_INTERVAL )
  {
    if ( $iNSR <= 5 )
    {
      &logInfo ( 20142, "<%s>: Not enough time left to do process checks", $sMyName);
      return 1;
    }
    else
    {
      if ( $sEntryPointName eq "offline" )
      {
        #-------------------------------------------------------------
        # We are in offline. Check whether WAS process is running.
        # If WAS process is running, sleep for 5 seconds.
        # Repeat above two steps till WAS process is gone.
        #-------------------------------------------------------------

        $iReturnCode = WebSphere::GetWASProcState($sWASHome,
                                                   $sWASNode,
                                                   $sServerName,
                                                   $sUser,
                                                   $sServerProfile,
                                                    $sServerType,
                                                   $sDMHostName,
                                                   $iDMPort,
                                                   $iMinClusterMember,
                                                   $iNSR);

        &logTrace (20143, "<%s>:  ReturnCode after GetWASProcState [%d] .", $sMyName, $iReturnCode);
        if ( 0 == $iReturnCode || 2 == $iReturnCode )
        {
          #-----------------------------
          # 0 - WAS pid is not running.
          # 2 - WAS pid a zombie. 
          #-----------------------------
          &logTrace(20144,"WAS process is not running.");
          return 1; 
        }
        else
        {
          &logInfo ( 20145, "<%s>: WAS process still running. Sleeping for 5 seconds", $sMyName);
          sleep CHECK_INTERVAL;
          $iNSR = $iDelayToTime - time;
        }
      } 
      elsif ( $sEntryPointName eq "online" )
      {
        #-------------------------------------------------------------
        # We are in online. Check whether WAS process is running.
        # If WAS process is NOT running, sleep for 5 seconds.
        # Repeat above two steps till WAS process is ONLINE.
        #-------------------------------------------------------------
        $iReturnCode = WebSphere::GetWASProcState($sWASHome,
                                                   $sWASNode,
                                                   $sServerName,
                                                   $sUser,
                                                   $sServerProfile,
                                                   $sServerType,
                                                   $sDMHostName,
                                                   $iDMPort,
                                                    $iMinClusterMember,
                                                   $iNSR);

        &logTrace (20146, "<%s>:  ReturnCode after GetWASProcState [%d] .", $sMyName, $iReturnCode);
        if ( ( 0 != $iReturnCode ) && ( 2 != $iReturnCode ) && ( 3 != $iReturnCode )  )
        {
          &logTrace ( 20147, "<%s>: WAS process is ONLINE", $sMyName);
          return 1;
        }
        else
        {
          &logInfo ( 20148, "<%s>: WAS process is not ONLINE. Sleeping for 5 seconds.", $sMyName);
          sleep CHECK_INTERVAL;
          $iNSR = $iDelayToTime - time;
        }
      }
    }
  }
  return 0;
}

=pod

 -------------------------------------------------------------------------
 Function : int RunSecondLevelMonitor( IN Integer SecondLevelMonitor )

 Description :  Determines whether now is the time to run the Second
 Level Monitoring checks for the WebSphere  resource. This subroutine 
 also maintains the slm counter file at the location :
 /tmp/.VRTS<ResourceType>/<ResourceName>.slm
 eg. /tmp/.VRTSWebSphere /WASRes.slm
 The file is updated every monitor cycle if SecondLevelMonitor is value
 is greater than 1. 

 Parameters :

  $iSecondLevelMonitor    SecondLevelMonitor attribute value

 Returns : Integer values as defined below.

  0  Second Level Monitor should not be run.
  1  Second Level Monitor need to be run.

 -------------------------------------------------------------------------

=cut

sub RunSecondLevelMonitor 
{
  #----------------
  # Get Arguments..
  #----------------
  my ( $iSecondLevelMonitor) = @_;
       
  my $sResourceType   = $ENV{'VCS_LOG_AGENT_NAME'};
  my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20308, "Subroutine <%s> called:
  SecondLevelMonitor        [%s]
  ResourceType              [%s]
  ResourceName              [%s]",
  $sMyName, $iSecondLevelMonitor, $sResourceType, $sResourceName );

  #---------------------------
  # Set Return Code to false..
  #---------------------------
  my $iReturnCode = 0; # Do not run SLM..

  #------------------------
  # Setup local variables..
  #------------------------
  my $sSLMDir  = "/tmp/.VRTS$sResourceType";
  my $sSLMFile = "$sSLMDir/$sResourceName.slm";
  my $iSLMCntr = undef;

  #--------------------------------------------
  # Validate SecondLevelMonitor trivial cases..
  #--------------------------------------------
  if ( 0 == $iSecondLevelMonitor ||
       1 == $iSecondLevelMonitor ) 
  {
    $iReturnCode = $iSecondLevelMonitor; # Assign..

    &logTrace (20309, "<%s>:Removing SLM file [%s] if it exists. Returning with [%s]", $sMyName, $sSLMFile, $iSecondLevelMonitor );

    unlink $sSLMFile if ( -e $sSLMFile ); # Remove SLM file..

    return $iReturnCode;
  }

  #----------------------------------
  # Create history folder if needed..
  #----------------------------------
  if ( -d $sSLMDir ) 
  {
    &logTrace ( 20310, "<%s>:SLM folder [%s] exists", $sMyName, $sSLMDir );
  }
  else 
  {
    mkdir ( $sSLMDir, 0777 ) ||
    &logWarn ( 20311, "<%s>:Failed to create SLM folder [%s]",
    $sMyName, $sSLMDir );
    chmod (0777,$sSLMDir );
    return 1; # Run SLM..
  }
  
  #--------------------------
  # Create SLM history file..
  #--------------------------
  if ( sysopen ( SLMFILE, $sSLMFile, O_RDWR|O_SYNC|O_CREAT, 0600 ) ) {
    &logTrace ( 20312,
    "<%s>:Opened SLM file [%s] successfully for read/write",
    $sMyName, $sSLMFile );
  }
  else {
    &logWarn ( 20313,
    "<%s>:Failed to open SLM file [%s] for read/write",
    $sMyName, $sSLMFile );
    return 1; # Run SLM..
  }
  
  #---------------
  # Read in file..
  #---------------
  ( $iSLMCntr ) = grep { s/\n// } <SLMFILE>; 

  #-----------------
  # Corrupted file..
  #-----------------
  if ( ! defined $iSLMCntr || $iSLMCntr < 0 ) {

    $iSLMCntr ||= ''; # Avoid undefs..

    &logTrace ( 20314,
    "<%s>:SLM file [%s] contains [%s]. Re-setting to [%s]",
    $sMyName, $sSLMFile, $iSLMCntr, $iSecondLevelMonitor );
    $iSLMCntr = $iSecondLevelMonitor; # Set to default..
  }
  
  #-----------------------------------------
  # Update counter, if less than requested..
  #-----------------------------------------
  if ( $iSecondLevelMonitor > $iSLMCntr ) {
    ++ $iSLMCntr;
  }
  else {
    $iSLMCntr = 1;    # Now's the time..
    $iReturnCode = 1; # Run SLM..
  }
  
  #------------------
  # Update the file..
  #------------------
  seek ( SLMFILE, 0, 0 ); # Rewind before update..

  &logTrace ( 20315,
  "<%s>:Updating SLM file [%s] with [%s]. Return code is [%s]",
  $sMyName, $sSLMFile, $iSLMCntr, $iReturnCode );

  print SLMFILE "$iSLMCntr\n"; # Update..
  
  close ( SLMFILE ) ||
  &logWarn ( 20316,
  "<%s>:Close failed on SLM file [%s] after update",
  $sMyName, $sSLMFile, $! );

  return $iReturnCode;
}

=pod

--------------------------------------------------------------------------------

 Function : bool GetPidsForIMFRegister(
                         IN String $sWASHome,
                         IN String $sWASNode,
                         IN String $sServerName,
                         IN String $sServerProfile ,
                         IN String $sServerType,
                         IN String DMHostName,
                         IN integer DMPort,
                         IN String $sUser,
                         IN Integer $iTimeOut,
                         OUT List $rliPids );

 Description : Start subroutine for online entry point. It attempts to start
               the WebSphere instance if its not running already.

 Parameters :

 $sWASHome            Absolute Path to WAS install home
 $sWASNode            Host on which WAS is configured
 $sServerName         WAS Server Name
 ServerProfile        WAS Server Profile name (only 6x)
 $sServerType         WAS Server Type
 DMHostName           Deployment Manager Host name
 DMPort               SOAP Connector port of Deployment Manager
 $sUser               Unix User for WAS instance
 $iTimeOut            Timeout value for this EP
 $rliPids             List of active pids to be registered for IMF

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true

--------------------------------------------------------------------------------

=cut

sub GetPidsForIMFRegister
{
  my ( $sWASHome,
       $sWASNode,
       $sServerName,
       $sServerProfile,
       $sServerType,
       $sDMHostName,
       $sDMPort,
       $sUser,
       $iTimeOut,
       $rliPids ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20332, "Subroutine <%s> called:
  WASHome                   [%s]
  WASNode                   [%s]
  ServerName                [%s]
  ServerProfile              [%s]
  ServerType                [%s]
  DMHostName                [%s]
  DMPort                    [%s]
  User                      [%s]
  TimeOut                   [%s]
  Pids                      [%s]",
  $sMyName, $sWASHome, $sWASNode, $sServerName, $sServerProfile, $sServerType,
  $sDMHostName, $sDMPort, $sUser, $iTimeOut, $rliPids );

  #-----------------------------------------------------------
  # First try to read the pid from pid file and check whether 
  # its running.
  #-----------------------------------------------------------
  my ( $sProfileDir, $sFullProfilePath, $iPid ) = ();

  if ( $sServerProfile ) {
    if ( $sServerProfile =~ m|^/| ) { 
      $sProfileDir = "${sServerProfile}/";
      $sFullProfilePath = "$sProfileDir";
    } 
    else {
      $sProfileDir = "profiles/${sServerProfile}/"; 
      $sFullProfilePath = "$sWASHome/$sProfileDir";
    }
  }

  my $sWASBinDir  = "${sProfileDir}bin";
  my $sWASLogDir  = "${sProfileDir}logs";
  my $sPidFile    = "${sFullProfilePath}logs/$sServerName/$sServerName.pid"; 
  my $iReturn = 0;
  my ( @hProcListHash, @sServerList ) = ();

  if ( $sServerType eq "Cluster" ){
    my $iProcessCount = Proc::GetLongProcessListHash(\@hProcListHash);
    if ( 0 == $iProcessCount )
    {
      #----------------------
      # No processes found.
      #----------------------
      &logError(20073,"<%s> : Error getting all running processes.",$sMyName);
      return 0;
    }
 
  $iReturn = GetClusterMembers( $sServerName,
                                $sServerProfile,
                                $sUser,
                                $sDMHostName,
                                $sDMPort,
                                $sWASBinDir,
                                $iTimeOut,
                                \@sServerList );
  if ( $iReturn ){
    &logError( 20278, "<%s>: Error getting list of cluster members.", $sMyName );
    return 0;
  }

  my $sServer = "";
  my $iRunningCount = 0;
  my $sOSType = Arch::GetOSType();
  foreach $sServer ( @sServerList ){
  #--------------------------------------
  # Form the filter to match WAS process
  #--------------------------------------
  my $sRegexWASName = GetServerNameRegex($sServer);
  $sRegexWASName =~ s/\s+/\\s+/g;
  my $sFilter = "^".$sWASHome."/.*/bin/.*java.*"."com.ibm.ws.runtime.WsServer".".*"."\\s+".$sRegexWASName."\\s*\$";
  if ( Arch::HPUX_OS == $sOSType )
  {
    $sFilter = "^".$sWASHome.".*/bin/.*java.*";
  }
  &logTrace(20074,"<%s>:Filter [%s] set.",$sMyName,$sFilter);
  my @hProcListHashMatched = ();

  my $iNumberOfMatches = Proc::FilterProcs( $sFilter,
                                            \@hProcListHash,
                                            \@hProcListHashMatched );
  if ( 0 >= $iNumberOfMatches )
  {
    $sFilter = "^".$sWASHome."/.*/bin/.*java.*$sRegexWASName";
    $iNumberOfMatches = Proc::FilterProcs( $sFilter,
                                           \@hProcListHash,
                                           \@hProcListHashMatched );

    &logTrace(20331,"<%s>: [%s] Processes found with filter [%s]",
      $sMyName , $iNumberOfMatches , $sFilter);
  }

  if ( 0 >= $iNumberOfMatches )
  {
    #--------------------
    # No matches found.
    #--------------------
    &logTrace(20075,"<%s>: No processes found for [%s] websphere server.",
      $sMyName, $sServer);
    $iReturn = 0;
  }
  else
  {
    #------------------------------------------
    # Resource's processes found to be running.
    #------------------------------------------
    &logTrace(20076,"<%s>: [%s] Processes found with filter [%s]",
      $sMyName , $iNumberOfMatches , $sFilter);
    $iRunningCount += 1;

    foreach my $hEachProcHash ( @hProcListHashMatched)
    {
      my $iTmpPPID = $hEachProcHash->{"PPID"};
      my $iTmpPID  = $hEachProcHash->{"PID"};
      my $iTmpCMD  = $hEachProcHash->{"COMMAND"};
      push(@$rliPids,$iTmpPID);
    }
  }
  &logTrace( 20279, "<%s>: Number of matching cluster members: [%s]. Pids list: [%s]", $sMyName, $iRunningCount, join (',', @$rliPids) );
}
    $iReturn = 1;
}
else{ 
  my $iReturn = open (FH,"$sPidFile") || 0;
  my $bDoRFLM = 0;

  if ( $iReturn ) {
    $iPid = <FH>;
    $iReturn = CheckWASProcessID($iPid,
                                  $sWASHome,
                                 $sWASNode,
                                 $sServerName,
                                 $sUser,
                                 $iTimeOut);

    if ( 1 == $iReturn )
    {
      #-----------------------------
      # Good pid. Register This Pid
      #-----------------------------
      push ( @$rliPids, $iPid );
    }
    elsif ( 3 == $iReturn )
    {
      #-------------------
      # Get Pid from RFLM
      #-------------------
      $bDoRFLM = 1;
    }
  }
  else {
    #-------------------------
    # Error opening pid file.
    #-------------------------
    $bDoRFLM = 1;
  }

  if ( $bDoRFLM ) {
    my $sOSType = Arch::GetOSType();
    if ( Arch::HPUX_OS != $sOSType )
    {
      $iReturn = IsWASProcessRunning($sWASHome,
                        $sWASNode,
                                     $sServerName,
                                     \@$rliPids);
    }
  }
}
  return $iReturn;
}

=pod

--------------------------------------------------------------------------------

 Function : bool GetClusterMembers ( IN String $sServerName,
                                     IN String $sServerProfile,
                                     IN String $sUser,
                                     IN String DMHostName,
                                     IN integer DMPort,
                                     IN String WASBinDirectory,
                                     IN Integer $iTimeOut,
                                     OUT reference ServerList );

 Description : This subroutine returns list of name of cluster members of a WAS Cluster.
               

 Parameters :

 $sServerName         WAS Server Name
 $sUser               Unix User for WAS instance
 ServerProfile        Deployment Manager profile name or profile path
 DMHostName           Deployment Manager Host name
 DMPort               SOAP Connector port of Deployment Manager
 WASBinDirectory      wsadmin location directory for Deployment manager
 $iTimeOut        Timeout value for this EP
 ServerList           List of names of cluster members of WAS Cluster


 Returns : boolean value in the set of {0,1} where
           0 Successfully retrieved list.
           1 Unsuccessful.

--------------------------------------------------------------------------------

=cut


sub GetClusterMembers{
  my ( $sServerName,
       $sServerProfile,
       $sUser,
       $sDMHostName,
       $iDMPort,
       $sWASBinDir,
       $iTimeOut,
       $sServerList ) = @_;

  my $sMyName = ( caller 0 )[3];
  &logTrace( 20280, "Subroutine <%s> called:
        ServerName       [%s]
        ServerProfile    [%s]
        User             [%s]
        DMHostName       [%s]
        DMPort           [%s]
        WASBinDir        [%s]
        TimeOut          [%s]
        ServerList       [%s]",
        $sMyName, $sServerName, $sServerProfile, $sUser, $sDMHostName, $iDMPort, $sWASBinDir, $iTimeOut, $sServerList );

  my ( $iReturn, $iReturnCode, $iSourceEnvFlag, $iExitCode, $bTimedOut ) = 0 ; 
  my $sEnvFile = "";
  my @lsCommandOut;
  my $iEntryPointStartTime = $^T;
  $iTimeOut = $iTimeOut - (time - $iEntryPointStartTime) - 3 ;
  
  my $sMonitorScript = "$sWASBinDir/".WebSphere::WAS_WSADMIN;
    #--------------------------------------------------------------------------------------------------
    #Command to query list of Ids cluster of cluster members
    #wsadmin.sh -lang jython -c "clusterName='hhCluster'; clusterId = AdminConfig.getid('/ServerCluster:' + clusterName + '/'); serverList = AdminConfig.list('ClusterMember', clusterId);
    #--------------------------------------------------------------------------------------------------
  my $sListMemberOptions = "-lang jython -conntype SOAP -host ".$sDMHostName." -port ".$iDMPort." -c \"clusterName=\'".$sServerName."\'; clusterId = AdminConfig.getid('/ServerCluster:' + clusterName + '/'); serverList = AdminConfig.list('ClusterMember', clusterId); serverList.replace('\\n', ',');\"";
  $iReturn = Sys::RunWithEnvCmdWithOutputWithTimeOut( $sEnvFile,
                                                         $iSourceEnvFlag,
                                                         $sMonitorScript,
                                                         $sListMemberOptions,
                                                         \$iExitCode,
                                                         $sUser,
                                                         \@lsCommandOut,
                                                         \$bTimedOut,
                                                         $iTimeOut,
                                                         "",
                                                         "$sWASBinDir" );
  if ( $iReturn ){
  #----------------------
  # Command got executed.
  #----------------------
    if ( $iExitCode == 0 ){
      &logTrace( 20379, "<%s>: [%s] command executed successfully.", $sMyName, $sMonitorScript );
      $iReturnCode = 0;
    }
    elsif ( $iExitCode == 1 ){
      &logError( 20382, "<%s>: Specified profile [%s] cannot be found or profile not linked correctly.", $sMyName, $sServerProfile );
      $iReturnCode = 1;
    }
    elsif ( $iExitCode == 103 ){
      &logError( 20383, "<%s>: Wrong Server Profile [%s] specified or Deployment Manager is not runnig or some error occured while executing [%s].", $sMyName, $sServerProfile, $sMonitorScript );
      $iReturnCode = 1;
    }
    else{
      &logError( 20384, "<%s>: Unknown error occured. Command [%s] with args [%s] returned exit code [%s]", $sMyName,$sMonitorScript,$sListMemberOptions, $iExitCode );
      $iReturnCode = 1;
    }
  }
  else{
    if ( $bTimedOut ){
    #-------------------
    # Command timed out.
    #-------------------
      &logError ( 20385,"<%s>:[%s] did not complete within [%s] seconds.", $sMyName, $sMonitorScript, $iTimeOut );
    }
    else
    {
    #-------------------
    # Command failed.
    #-------------------
      &logError ( 20386,"<%s>: Command [%s] failed.", $sMyName,$sMonitorScript );
    }
    $iReturnCode = 1;
  }

  if ( $iExitCode == 0 ){
    #---------------------------------------------
    # Sample command output:
    # WASX7209I: Connected to process "dmgr" on node Dmgr02-node using SOAP connector;  The type of process is: DeploymentManager
    # 'horizontal_cluster_member5(cells/Dmgr02-cell/clusters/hhCluster|cluster.xml#ClusterMember_1423657346180),managed_server3(cells/Dmgr02-cell/clusters/hhCluster|cluster.xml#ClusterMember_1423657345146),managed_server4(cells/Dmgr02-cell/clusters/hhCluster|cluster.xml#ClusterMember_1423657345686),server3(cells/Dmgr02-cell/clusters/hhCluster|cluster.xml#ClusterMember_1423657344437)'
    #---------------------------------------------
    my $sServerIds = $lsCommandOut[1];
    $sServerIds =~ s/\'//g;
    my @lsServers = split(/,/, $sServerIds);

    my $sServer;
    foreach $sServer ( @lsServers ){
      push @$sServerList, substr($sServer, 0, index($sServer, '('));
    }
    &logTrace( 20281, "<%s>: List of cluster members found: [%s]", $sMyName, join( ', ', @$sServerList ) );
  }

  return $iReturnCode;
}

=pod
--------------------------------------------------------------------------------
 Function: bool GetNodeId( OUT Reference NodeId )

 Description: This subroutine returns VCS nodeid of host where agent is running

 Parameters:
 NodeId            Node name of current host 

 Returns : boolean value in the set of {0,1} where
           0 Successfully retrieved node name.
           1 Unsuccessful.
--------------------------------------------------------------------------------
=cut

sub GetNodeId{
  
  my $sCurrentHost = shift;
  my $sMyName = ( caller 0 )[3];
  &logTrace( 20275, "Subroutine <%s> called:
                     CurrentHost [%s]", $sMyName, $sCurrentHost );
  my $iExitCode = 0;
  my $sCommand = $sCmdHasys;
  my $sCommandArgs = " -nodeid ";
  $$sCurrentHost = readpipe ( "$sCommand $sCommandArgs" );
  if ( $iExitCode = $? >> 8 ) {
    &logError ( 20283,"<%s>: Cmd [%s] with args [%s] failed with exit code [%s].", $sMyName, $sCommand, $sCommandArgs, $iExitCode );
    return 1;
  }

  chomp $$sCurrentHost;
  $$sCurrentHost =~ s/.*Node name: //;
  $$sCurrentHost =~ s/ Node ID: .*//;
  &logTrace( 20282,"<%s>: Current host is [%s]", $sMyName, $$sCurrentHost );

  return 0;
}


1; #so perl knows its included ok
